@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# Copyright (c) 2012-2015 CORE Security Technologies
+# Copyright (c) 2012-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -13,19 +13,18 @@
 # Reference for:
 #  DCE/RPC [MS-LSAT]
 
-import socket
-import string
 import sys
-import types
 import logging
+import argparse
+import codecs
 
-from impacket import uuid, ntlm, version
 from impacket.examples import logger
+from impacket import version
 from impacket.dcerpc.v5 import transport, lsat, lsad
-from impacket.dcerpc.v5.ndr import NULL
 from impacket.dcerpc.v5.samr import SID_NAME_USE
 from impacket.dcerpc.v5.dtypes import MAXIMUM_ALLOWED
-import argparse
+from impacket.dcerpc.v5.rpcrt import DCERPCException
+
 
 class LSALookupSid:
     KNOWN_PROTOCOLS = {
@@ -54,7 +53,6 @@ class LSALookupSid:
         logging.info('Brute forcing SIDs at %s' % addr)
 
         # Try all requested protocols until one works.
-        entries = []
         for protocol in self.__protocols:
             protodef = LSALookupSid.KNOWN_PROTOCOLS[protocol]
             port = protodef[1]
@@ -69,7 +67,7 @@ class LSALookupSid:
                 rpctransport.set_credentials(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash)
 
             try:
-                entries = self.__bruteForce(rpctransport, self.__maxRid)
+                self.__bruteForce(rpctransport, self.__maxRid)
             except Exception, e:
                 #import traceback
                 #print traceback.print_exc()
@@ -112,10 +110,10 @@ class LSALookupSid:
 
             sids = list()
             for i in xrange(soFar, soFar+sidsToCheck):
-                sids.append(domainSid + '-%d' % (i))
+                sids.append(domainSid + '-%d' % i)
             try:
-                request = lsat.hLsarLookupSids(dce, policyHandle, sids,lsat.LSAP_LOOKUP_LEVEL.LsapLookupWksta)
-            except Exception, e:
+                lsat.hLsarLookupSids(dce, policyHandle, sids,lsat.LSAP_LOOKUP_LEVEL.LsapLookupWksta)
+            except DCERPCException, e:
                 if str(e).find('STATUS_NONE_MAPPED') >= 0:
                     soFar += SIMULTANEOUS
                     continue
@@ -126,7 +124,9 @@ class LSALookupSid:
 
             for n, item in enumerate(resp['TranslatedNames']['Names']):
                 if item['Use'] != SID_NAME_USE.SidTypeUnknown:
-                    print "%d: %s\\%s (%s)" % (soFar+n, resp['ReferencedDomains']['Domains'][item['DomainIndex']]['Name'], item['Name'], SID_NAME_USE.enumItems(item['Use']).name)
+                    print "%d: %s\\%s (%s)" % (
+                    soFar + n, resp['ReferencedDomains']['Domains'][item['DomainIndex']]['Name'], item['Name'],
+                    SID_NAME_USE.enumItems(item['Use']).name)
             soFar += SIMULTANEOUS
 
         dce.disconnect()
@@ -136,6 +136,12 @@ class LSALookupSid:
 
 # Process command-line arguments.
 if __name__ == '__main__':
+    # Init the example's logger theme
+    logger.init()
+    # Explicitly changing the stdout encoding format
+    if sys.stdout.encoding is None:
+        # Output is redirected to a file
+        sys.stdout = codecs.getwriter('utf8')(sys.stdout)
     print version.BANNER
 
     parser = argparse.ArgumentParser()
@@ -155,7 +161,14 @@ if __name__ == '__main__':
     options = parser.parse_args()
 
     import re
-    domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(options.target).groups('')
+
+    domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(
+        options.target).groups('')
+
+    #In case the password contains '@'
+    if '@' in address:
+        password = password + '@' + address.rpartition('@')[0]
+        address = address.rpartition('@')[2]
 
     if domain is None:
         domain = ''
@@ -167,5 +180,5 @@ if __name__ == '__main__':
     lookup = LSALookupSid(username, password, domain, options.protocol, options.hashes, options.maxRid)
     try:
         lookup.dump(address)
-    except Exception, e:
+    except:
         pass
