@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2015 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -15,16 +15,19 @@
 #
 
 
-from impacket import version, tds
-from impacket.examples import logger
 import argparse
 import sys
 import string
+import logging
+
+from impacket.examples import logger
+from impacket import version, tds
 
 if __name__ == '__main__':
-    import cmd
 
     print version.BANNER
+    # Init the example's logger theme
+    logger.init()
 
     parser = argparse.ArgumentParser(add_help = True, description = "Asks the remote host for its running MSSQL Instances.")
 
