@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2015 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -17,10 +17,11 @@
 #        authentication process (check [MS-CSSP] section 3.1.5)
 #
 
-from impacket.structure import Structure
-from impacket.spnego import *
 from struct import pack, unpack
+
 from impacket.examples import logger
+from impacket.structure import Structure
+from impacket.spnego import GSSAPI, ASN1_SEQUENCE, ASN1_OCTET_STRING, asn1decode, asn1encode
 
 TDPU_CONNECTION_REQUEST  = 0xe0
 TPDU_CONNECTION_CONFIRM  = 0xd0
@@ -274,7 +275,6 @@ if __name__ == '__main__':
     import socket
     import argparse
     import sys
-    import ssl
     import logging
     from binascii import a2b_hex
     from Crypto.Cipher import ARC4
@@ -287,10 +287,10 @@ if __name__ == '__main__':
         sys.exit(1)
     
 
-    class SPNEGOCipher():
+    class SPNEGOCipher:
         def __init__(self, flags, randomSessionKey):
             self.__flags = flags
-            if self.__flags & ntlm.NTLMSSP_NTLM2_KEY:
+            if self.__flags & ntlm.NTLMSSP_NEGOTIATE_EXTENDED_SESSIONSECURITY:
                 self.__clientSigningKey = ntlm.SIGNKEY(self.__flags, randomSessionKey)
                 self.__serverSigningKey = ntlm.SIGNKEY(self.__flags, randomSessionKey,"Server")
                 self.__clientSealingKey = ntlm.SEALKEY(self.__flags, randomSessionKey)
@@ -312,7 +312,7 @@ if __name__ == '__main__':
             self.__sequence = 0
 
         def encrypt(self, plain_data):
-            if self.__flags & ntlm.NTLMSSP_NTLM2_KEY:
+            if self.__flags & ntlm.NTLMSSP_NEGOTIATE_EXTENDED_SESSIONSECURITY:
                 # When NTLM2 is on, we sign the whole pdu, but encrypt just
                 # the data, not the dcerpc header. Weird..
                 sealedMessage, signature =  ntlm.SEAL(self.__flags, 
@@ -336,7 +336,7 @@ if __name__ == '__main__':
             return signature, sealedMessage
 
         def decrypt(self, answer):
-            if self.__flags & ntlm.NTLMSSP_NTLM2_KEY:
+            if self.__flags & ntlm.NTLMSSP_NEGOTIATE_EXTENDED_SESSIONSECURITY:
                 # TODO: FIX THIS, it's not calculating the signature well
                 # Since I'm not testing it we don't care... yet
                 answer, signature =  ntlm.SEAL(self.__flags, 
@@ -458,7 +458,7 @@ if __name__ == '__main__':
        # in the pubKeyAuth field.
 
        # Last SPNEGO token calculation
-       ntlmChallenge = ntlm.NTLMAuthChallenge(ts_request['NegoData'])
+       #ntlmChallenge = ntlm.NTLMAuthChallenge(ts_request['NegoData'])
        type3, exportedSessionKey = ntlm.getNTLMSSPType3(auth, ts_request['NegoData'], username, password, domain, lmhash, nthash, use_ntlmv2 = True)
 
        # Get server public key
@@ -541,6 +541,8 @@ if __name__ == '__main__':
        tls.close()
        logging.info("Access Granted")
 
+    # Init the example's logger theme
+    logger.init()
     print version.BANNER
 
     parser = argparse.ArgumentParser(add_help = True, description = "Test whether an account is valid on the target host using the RDP protocol.")
@@ -559,6 +561,11 @@ if __name__ == '__main__':
     import re
     domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(options.target).groups('')
 
+    #In case the password contains '@'
+    if '@' in address:
+        password = password + '@' + address.rpartition('@')[0]
+        address = address.rpartition('@')[2]
+
     if domain is None:
         domain = ''
 
