@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2015 CORE Security Technologies)
+# Copyright (c) 2003-2016 CORE Security Technologies)
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -13,13 +13,15 @@
 #  winregistry.py
 #
 
-import impacket
-from impacket.examples import logger
-from impacket import version
-from impacket import winregistry
 import sys
 import argparse
 import ntpath
+from binascii import unhexlify, hexlify
+
+from impacket.examples import logger
+from impacket import version
+from impacket import winregistry
+
 
 def bootKey(reg):
     baseClass = 'ControlSet001\\Control\\Lsa\\'
@@ -27,7 +29,7 @@ def bootKey(reg):
     tmpKey = ''
 
     for key in keys:
-        tmpKey = tmpKey + reg.getClass(baseClass + key).decode('utf-16le')[:8].decode('hex')
+        tmpKey = tmpKey + unhexlify(reg.getClass(baseClass + key).decode('utf-16le')[:8])
 
     transforms = [ 8, 5, 4, 2, 11, 9, 13, 3, 0, 6, 1, 12, 14, 10, 15, 7 ]
 
@@ -35,7 +37,7 @@ def bootKey(reg):
     for i in xrange(len(tmpKey)):
         syskey += tmpKey[transforms[i]]
 
-    print syskey.encode('hex')
+    print hexlify(syskey)
 
 def getClass(reg, className):
     regKey = ntpath.dirname(className)
@@ -77,7 +79,7 @@ def enumValues(reg, searchKey):
     values = reg.enumValues(key)
 
     for value in values:
-        print "  %-30s: " % (value),
+        print "  %-30s: " % value,
         data = reg.getValue('%s\\%s'%(searchKey,value))
         # Special case for binary string.. so it looks better formatted
         if data[0] == winregistry.REG_BINARY:
@@ -99,7 +101,7 @@ def enumKey(reg, searchKey, isRecursive, indent='  '):
         print "%s%s" %(indent, key)
         if isRecursive is True:
             if searchKey == '\\':
-                enumKey(reg, '\\%s'%(key),isRecursive,indent+'  ')
+                enumKey(reg, '\\%s'%key,isRecursive,indent+'  ')
             else:
                 enumKey(reg, '%s\\%s'%(searchKey,key),isRecursive,indent+'  ')
 
@@ -108,6 +110,8 @@ def walk(reg, keyName):
 
 
 def main():
+    # Init the example's logger theme
+    logger.init()
     print version.BANNER
 
     parser = argparse.ArgumentParser(add_help = True, description = "Reads data from registry hives.")
