@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2015 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -16,18 +16,19 @@
 #
 
 import sys
-import string
 import time
 import logging
-from impacket.examples import logger 
-from impacket import smb, version, smb3, nt_errors, version
+import argparse
+import cmd
+import os
+
+from impacket.examples import logger
+from impacket import version
 from impacket.dcerpc.v5 import samr, transport, srvs
 from impacket.dcerpc.v5.dtypes import NULL
 from impacket.smbconnection import *
-import argparse
-import ntpath
-import cmd
-import os
+
+
 # If you wanna have readline like functionality in Windows, install pyreadline
 try:
   import pyreadline as readline
@@ -45,6 +46,7 @@ class MiniImpacketShell(cmd.Cmd):
         self.pwd = ''
         self.share = None
         self.loggedIn = True
+        self.last_output = None
         self.completion = []
 
     def emptyline(self):
@@ -83,6 +85,7 @@ class MiniImpacketShell(cmd.Cmd):
  shares - list available shares
  use {sharename} - connect to an specific share
  cd {path} - changes the current directory to {path}
+ lcd {path} - changes the current local directory to {path}
  pwd - shows current remote directory
  password - changes the user password, the new password will be prompted for input
  ls {wildcard} - lists all the files in the current directory
@@ -108,7 +111,7 @@ class MiniImpacketShell(cmd.Cmd):
         dce = rpctransport.get_dce_rpc()
         dce.connect()                     
         dce.bind(samr.MSRPC_UUID_SAMR)
-        resp = samr.hSamrUnicodeChangePasswordUser2(dce, '\x00', self.username, self.password, newPassword, self.lmhash, self.nthash)
+        samr.hSamrUnicodeChangePasswordUser2(dce, '\x00', self.username, self.password, newPassword, self.lmhash, self.nthash)
         self.password = newPassword
         self.lmhash = None
         self.nthash = None
@@ -245,7 +248,7 @@ class MiniImpacketShell(cmd.Cmd):
             logging.error("No connection open")
             return
         self.smb.logoff()
-        del(self.smb)
+        del self.smb
         self.share = None
         self.smb = None
         self.tid = None
@@ -284,7 +287,9 @@ class MiniImpacketShell(cmd.Cmd):
         resp = srvs.hNetrSessionEnum(dce, NULL, NULL, 10)
 
         for session in resp['InfoStruct']['SessionInfo']['Level10']['Buffer']:
-            print "host: %15s, user: %5s, active: %5d, idle: %5d" % (session['sesi10_cname'][:-1], session['sesi10_username'][:-1], session['sesi10_time'], session['sesi10_idle_time'])
+            print "host: %15s, user: %5s, active: %5d, idle: %5d" % (
+            session['sesi10_cname'][:-1], session['sesi10_username'][:-1], session['sesi10_time'],
+            session['sesi10_idle_time'])
 
     def do_shares(self, line):
         if self.loggedIn is False:
@@ -319,20 +324,22 @@ class MiniImpacketShell(cmd.Cmd):
         self.pwd = ntpath.normpath(self.pwd)
         # Let's try to open the directory to see if it's valid
         try:
-            fid = self.smb.openFile(self.tid, self.pwd)
+            fid = self.smb.openFile(self.tid, self.pwd, creationOption = FILE_DIRECTORY_FILE \
+                                    , desiredAccess = FILE_READ_DATA | FILE_LIST_DIRECTORY \
+                                    , shareMode = FILE_SHARE_READ | FILE_SHARE_WRITE \
+                                    )
             self.smb.closeFile(self.tid,fid)
-            self.pwd = oldpwd
-            logging.error("Invalid directory")
-        except SessionError, e:
-            if e.getErrorCode() == nt_errors.STATUS_FILE_IS_A_DIRECTORY: 
-               pass
-            else:
-               self.pwd = oldpwd
-               raise
-        except Exception, e:
+        except SessionError:
             self.pwd = oldpwd
             raise
 
+    def do_lcd(self, s):
+        print s
+        if s == '':
+           print os.getcwd()
+        else:
+           os.chdir(s)
+
     def do_pwd(self,line):
         if self.loggedIn is False:
             logging.error("Not logged in")
@@ -354,9 +361,11 @@ class MiniImpacketShell(cmd.Cmd):
         pwd = string.replace(pwd,'/','\\')
         pwd = ntpath.normpath(pwd)
         for f in self.smb.listPath(self.share, pwd):
-           if display is True:
-               print "%crw-rw-rw- %10d  %s %s" % ('d' if f.is_directory() > 0 else '-', f.get_filesize(),  time.ctime(float(f.get_mtime_epoch())) ,f.get_longname() )
-           self.completion.append((f.get_longname(),f.is_directory()))
+            if display is True:
+                print "%crw-rw-rw- %10d  %s %s" % (
+                'd' if f.is_directory() > 0 else '-', f.get_filesize(), time.ctime(float(f.get_mtime_epoch())),
+                f.get_longname())
+            self.completion.append((f.get_longname(), f.is_directory()))
 
 
     def do_rm(self, filename):
@@ -437,6 +446,8 @@ class MiniImpacketShell(cmd.Cmd):
         self.do_logoff(line)
 
 def main():
+    # Init the example's logger theme
+    logger.init()
     print version.BANNER
     parser = argparse.ArgumentParser(add_help = True, description = "SMB client implementation.")
 
@@ -450,6 +461,8 @@ def main():
     group.add_argument('-no-pass', action="store_true", help='don\'t ask for password (useful for -k)')
     group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file (KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the ones specified in the command line')
     group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication (128 or 256 bits)')
+    group.add_argument('-dc-ip', action='store', metavar="ip address",
+                       help='IP Address of the domain controller. If ommited it use the domain part (FQDN) specified in the target parameter')
 
     if len(sys.argv)==1:
         parser.print_help()
@@ -463,7 +476,13 @@ def main():
         logging.getLogger().setLevel(logging.INFO)
 
     import re
-    domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(options.target).groups('')
+    domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(
+        options.target).groups('')
+
+    #In case the password contains '@'
+    if '@' in address:
+        password = password + '@' + address.rpartition('@')[0]
+        address = address.rpartition('@')[2]
 
     if domain is None:
         domain = ''
@@ -484,7 +503,7 @@ def main():
     try: 
         smbClient = SMBConnection(address, address)
         if options.k is True:
-            smbClient.kerberosLogin(username, password, domain, lmhash, nthash, options.aesKey )
+            smbClient.kerberosLogin(username, password, domain, lmhash, nthash, options.aesKey, options.dc_ip )
         else:
             smbClient.login(username, password, domain, lmhash, nthash)
 
