@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2015 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -32,17 +32,14 @@ import sys
 import os
 import cmd
 import argparse
-import random
-import string
-import time
 import ConfigParser
 import logging
 from threading import Thread
 
+from impacket.examples import logger
 from impacket import version, smbserver
 from impacket.smbconnection import *
 from impacket.dcerpc.v5 import transport, scmr
-from impacket.examples import logger
 
 OUTPUT_FILENAME = '__output'
 BATCH_FILENAME  = 'execute.bat'
@@ -52,6 +49,7 @@ DUMMY_SHARE     = 'TMP'
 class SMBServer(Thread):
     def __init__(self):
         Thread.__init__(self)
+        self.smb = None
 
     def cleanup_server(self):
         logging.info('Cleaning up..')
@@ -112,11 +110,11 @@ class CMDEXEC:
         '445/SMB': (r'ncacn_np:%s[\pipe\svcctl]', 445),
         }
 
-
-    def __init__(self, protocols = None, 
-                 username = '', password = '', domain = '', hashes = None, aesKey = None, doKerberos = None, mode = None, share = None):
+    def __init__(self, protocols=None,
+                 username='', password='', domain='', hashes=None, aesKey=None, doKerberos=None, kdcHost=None,
+                 mode=None, share=None):
         if not protocols:
-            protocols = PSEXEC.KNOWN_PROTOCOLS.keys()
+            protocols = CMDEXEC.KNOWN_PROTOCOLS.keys()
 
         self.__username = username
         self.__password = password
@@ -127,8 +125,10 @@ class CMDEXEC:
         self.__nthash = ''
         self.__aesKey = aesKey
         self.__doKerberos = doKerberos
+        self.__kdcHost = kdcHost
         self.__share = share
         self.__mode  = mode
+        self.shell = None
         if hashes is not None:
             self.__lmhash, self.__nthash = hashes.split(':')
 
@@ -149,8 +149,9 @@ class CMDEXEC:
                rpctransport.preferred_dialect(SMB_DIALECT)
             if hasattr(rpctransport, 'set_credentials'):
                 # This method exists only for selected protocol sequences.
-                rpctransport.set_credentials(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash, self.__aesKey)
-            rpctransport.set_kerberos(self.__doKerberos)
+                rpctransport.set_credentials(self.__username, self.__password, self.__domain, self.__lmhash,
+                                             self.__nthash, self.__aesKey)
+            rpctransport.set_kerberos(self.__doKerberos, self.__kdcHost)
 
             self.shell = None
             try:
@@ -176,7 +177,7 @@ class RemoteShell(cmd.Cmd):
         cmd.Cmd.__init__(self)
         self.__share = share
         self.__mode = mode
-        self.__output = '\\Windows\\Temp\\' + OUTPUT_FILENAME 
+        self.__output = '\\\\127.0.0.1\\' + self.__share + '\\' + OUTPUT_FILENAME
         self.__batchFile = '%TEMP%\\' + BATCH_FILENAME 
         self.__outputBuffer = ''
         self.__command = ''
@@ -211,7 +212,7 @@ class RemoteShell(cmd.Cmd):
         try:
            self.__scmr = self.__rpc.get_dce_rpc()
            self.__scmr.connect() 
-           self.__scmr.bind(svcctl.MSRPC_UUID_SVCCTL)
+           self.__scmr.bind(scmr.MSRPC_UUID_SCMR)
            resp = scmr.hROpenSCManagerW(self.__scmr)
            self.__scHandle = resp['lpScHandle']
            resp = scmr.hROpenServiceW(self.__scmr, self.__scHandle, self.__serviceName)
@@ -219,7 +220,7 @@ class RemoteShell(cmd.Cmd):
            scmr.hRDeleteService(self.__scmr, service)
            scmr.hRControlService(self.__scmr, service, scmr.SERVICE_CONTROL_STOP)
            scmr.hRCloseServiceHandle(self.__scmr, service)
-        except Exception, e:
+        except:
            pass
 
     def do_shell(self, s):
@@ -254,8 +255,8 @@ class RemoteShell(cmd.Cmd):
             self.__outputBuffer += data
 
         if self.__mode == 'SHARE':
-            self.transferClient.getFile(self.__share, self.__output, output_callback)
-            self.transferClient.deleteFile(self.__share, self.__output)
+            self.transferClient.getFile(self.__share, OUTPUT_FILENAME, output_callback)
+            self.transferClient.deleteFile(self.__share, OUTPUT_FILENAME)
         else:
             fd = open(SMBSERVER_DIR + '/' + OUTPUT_FILENAME,'r')
             output_callback(fd.read())
@@ -263,11 +264,13 @@ class RemoteShell(cmd.Cmd):
             os.unlink(SMBSERVER_DIR + '/' + OUTPUT_FILENAME)
 
     def execute_remote(self, data):
-        command = self.__shell + 'echo ' + data + ' ^> ' + self.__output + ' 2^>^&1 > ' + self.__batchFile + ' & ' + self.__shell + self.__batchFile 
+        command = self.__shell + 'echo ' + data + ' ^> ' + self.__output + ' 2^>^&1 > ' + self.__batchFile + ' & ' + \
+                  self.__shell + self.__batchFile
         if self.__mode == 'SERVER':
             command += ' & ' + self.__copyBack
         command += ' & ' + 'del ' + self.__batchFile 
 
+        logging.debug('Executing %s' % command)
         resp = scmr.hRCreateServiceW(self.__scmr, self.__scHandle, self.__serviceName, self.__serviceName, lpBinaryPathName=command)
         service = resp['lpServiceHandle']
 
@@ -287,6 +290,8 @@ class RemoteShell(cmd.Cmd):
 
 # Process command-line arguments.
 if __name__ == '__main__':
+    # Init the example's logger theme
+    logger.init()
     print version.BANNER
 
     parser = argparse.ArgumentParser()
@@ -304,6 +309,7 @@ if __name__ == '__main__':
     group.add_argument('-no-pass', action="store_true", help='don\'t ask for password (useful for -k)')
     group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file (KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the ones specified in the command line')
     group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication (128 or 256 bits)')
+    group.add_argument('-dc-ip', action='store',metavar = "ip address",  help='IP Address of the domain controller. If ommited it use the domain part (FQDN) specified in the target parameter')
 
  
     if len(sys.argv)==1:
@@ -320,6 +326,11 @@ if __name__ == '__main__':
     import re
     domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(options.target).groups('')
 
+    #In case the password contains '@'
+    if '@' in address:
+        password = password + '@' + address.rpartition('@')[0]
+        address = address.rpartition('@')[2]
+
     if domain is None:
         domain = ''
 
@@ -331,7 +342,8 @@ if __name__ == '__main__':
         options.k = True
 
     try:
-        executer = CMDEXEC(options.protocol, username, password, domain, options.hashes, options.aesKey, options.k, options.mode, options.share)
+        executer = CMDEXEC(options.protocol, username, password, domain, options.hashes, options.aesKey, options.k,
+                           options.dc_ip, options.mode, options.share)
         executer.run(address)
     except Exception, e:
         logging.critical(str(e))
