@@ -23,14 +23,13 @@
 #  ImpactDecoder.
 
 import sys
-import string
 from threading import Thread
-
 import pcapy
 from pcapy import findalldevs, open_live
-import impacket
+
 from impacket.ImpactDecoder import EthDecoder, LinuxSLLDecoder
 
+
 class DecoderThread(Thread):
     def __init__(self, pcapObj):
         # Query the type of the link and instantiate a decoder accordingly.
