@@ -19,15 +19,13 @@
 #  ImpactDecoder.
 
 import sys
-import string
 from exceptions import Exception
-from threading import Thread
-
 import pcapy
 from pcapy import open_offline
-import impacket
+
 from impacket.ImpactDecoder import EthDecoder, LinuxSLLDecoder
 
+
 class Connection:
     """This class can be used as a key in a dictionary to select a connection
     given a pair of peers. Two connections are considered the same if both
