@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2015 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -24,20 +24,22 @@ import os
 import cmd
 import argparse
 import time
-import ntpath
 import logging
+import string
+import ntpath
 
-from impacket import version, ntlm
-from impacket.smbconnection import *
+from impacket.examples import logger
+from impacket import version
+from impacket.smbconnection import SMBConnection, SMB_DIALECT, SMB2_DIALECT_002, SMB2_DIALECT_21
 from impacket.dcerpc.v5.dcomrt import DCOMConnection
 from impacket.dcerpc.v5.dcom import wmi
 from impacket.dcerpc.v5.dtypes import NULL
-from impacket.examples import logger
 
-OUTPUT_FILENAME = '__'
+OUTPUT_FILENAME = '__' + str(time.time())
 
 class WMIEXEC:
-    def __init__(self, command = '', username = '', password = '', domain = '', hashes = None, aesKey = None, share = None, noOutput=False, doKerberos=False):
+    def __init__(self, command='', username='', password='', domain='', hashes=None, aesKey=None, share=None,
+                 noOutput=False, doKerberos=False, kdcHost=None):
         self.__command = command
         self.__username = username
         self.__password = password
@@ -48,6 +50,8 @@ class WMIEXEC:
         self.__share = share
         self.__noOutput = noOutput
         self.__doKerberos = doKerberos
+        self.__kdcHost = kdcHost
+        self.shell = None
         if hashes is not None:
             self.__lmhash, self.__nthash = hashes.split(':')
 
@@ -57,7 +61,8 @@ class WMIEXEC:
             if self.__doKerberos is False:
                 smbConnection.login(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash)
             else:
-                smbConnection.kerberosLogin(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash, self.__aesKey)
+                smbConnection.kerberosLogin(self.__username, self.__password, self.__domain, self.__lmhash,
+                                            self.__nthash, self.__aesKey, kdcHost=self.__kdcHost)
 
             dialect = smbConnection.getDialect()
             if dialect == SMB_DIALECT:
@@ -71,15 +76,16 @@ class WMIEXEC:
         else:
             smbConnection = None
 
-        dcom = DCOMConnection(addr, self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash, self.__aesKey, oxidResolver = True, doKerberos=self.__doKerberos)
-        iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login,wmi.IID_IWbemLevel1Login)
-        iWbemLevel1Login = wmi.IWbemLevel1Login(iInterface)
-        iWbemServices= iWbemLevel1Login.NTLMLogin('//./root/cimv2', NULL, NULL)
-        iWbemLevel1Login.RemRelease()
-
-        win32Process,_ = iWbemServices.GetObject('Win32_Process')
-
+        dcom = DCOMConnection(addr, self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash,
+                              self.__aesKey, oxidResolver=True, doKerberos=self.__doKerberos, kdcHost=self.__kdcHost)
         try:
+            iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login,wmi.IID_IWbemLevel1Login)
+            iWbemLevel1Login = wmi.IWbemLevel1Login(iInterface)
+            iWbemServices= iWbemLevel1Login.NTLMLogin('//./root/cimv2', NULL, NULL)
+            iWbemLevel1Login.RemRelease()
+
+            win32Process,_ = iWbemServices.GetObject('Win32_Process')
+
             self.shell = RemoteShell(self.__share, win32Process, smbConnection)
             if self.__command != ' ':
                 self.shell.onecmd(self.__command)
@@ -135,7 +141,10 @@ class RemoteShell(cmd.Cmd):
         if s == '':
             print os.getcwd()
         else:
-            os.chdir(s)
+            try:
+                os.chdir(s)
+            except Exception, e:
+                logging.error(str(e))
 
     def do_get(self, src_path):
         try:
@@ -239,7 +248,7 @@ class RemoteShell(cmd.Cmd):
         command = self.__shell + data 
         if self.__noOutput is False:
             command += ' 1> ' + '\\\\127.0.0.1\\%s' % self.__share + self.__output  + ' 2>&1'
-        obj = self.__win32Process.Create(command, self.__pwd, None)
+        self.__win32Process.Create(command, self.__pwd, None)
         self.get_output()
 
     def send_data(self, data):
@@ -250,6 +259,8 @@ class RemoteShell(cmd.Cmd):
 
 # Process command-line arguments.
 if __name__ == '__main__':
+    # Init the example's logger theme
+    logger.init()
     print version.BANNER
 
     parser = argparse.ArgumentParser(add_help = True, description = "Executes a semi-interactive shell using Windows Management Instrumentation.")
@@ -266,8 +277,8 @@ if __name__ == '__main__':
     group.add_argument('-no-pass', action="store_true", help='don\'t ask for password (useful for -k)')
     group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file (KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the ones specified in the command line')
     group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication (128 or 256 bits)')
+    group.add_argument('-dc-ip', action='store',metavar = "ip address",  help='IP Address of the domain controller. If ommited it use the domain part (FQDN) specified in the target parameter')
 
- 
     if len(sys.argv)==1:
         parser.print_help()
         sys.exit(1)
@@ -284,7 +295,14 @@ if __name__ == '__main__':
         logging.getLogger().setLevel(logging.INFO)
 
     import re
-    domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(options.target).groups('')
+
+    domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(
+        options.target).groups('')
+
+    #In case the password contains '@'
+    if '@' in address:
+        password = password + '@' + address.rpartition('@')[0]
+        address = address.rpartition('@')[2]
 
     try:
         if domain is None:
@@ -297,7 +315,8 @@ if __name__ == '__main__':
         if options.aesKey is not None:
             options.k = True
 
-        executer = WMIEXEC(' '.join(options.command), username, password, domain, options.hashes, options.aesKey, options.share, options.nooutput, options.k)
+        executer = WMIEXEC(' '.join(options.command), username, password, domain, options.hashes, options.aesKey,
+                           options.share, options.nooutput, options.k, options.dc_ip)
         executer.run(address)
     except (Exception, KeyboardInterrupt), e:
         #import traceback
