@@ -1,17 +1,16 @@
-# Copyright (c) 2003-2012 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 # 
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 # 
-# $Id: ICMP6.py 1142 2014-02-17 17:56:43Z bethus@gmail.com $
-#
+
+import array
+import struct
 
 from ImpactPacket import Header, Data
 from IP6_Address import IP6_Address
-#from impacket import ImpactPacket
-from IP6 import IP6
-import array, struct
+
 
 class ICMP6(Header):    
     #IP Protocol number for ICMP6
