@@ -1,24 +1,30 @@
-# Copyright (c) 2003-2013 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: dtypes.py 1187 2014-04-03 15:01:43Z bethus@gmail.com $
-#
-# Author: Alberto Solino
+# Author: Alberto Solino (@agsolino)
 #
 # Description:
 #   [MS-DTYP] Interface mini implementation
 #
-import random
-from struct import pack, unpack
-from impacket.dcerpc.v5.ndr import NDRULONG, NDRUHYPER, NDRUSMALL, NDRSHORT, NDRLONG, NDRPOINTER, NDRUniConformantArray, NDRUniFixedArray, NDR, NDRHYPER, NDRSMALL, NDRPOINTERNULL, NDRSTRUCT, NULL, NDRUSMALL, NDRBOOLEAN, NDRUSHORT, NDRFLOAT, NDRDOUBLEFLOAT
+from struct import pack
+
+from impacket.dcerpc.v5.ndr import NDRULONG, NDRUHYPER, NDRSHORT, NDRLONG, NDRPOINTER, NDRUniConformantArray, \
+    NDRUniFixedArray, NDR, NDRHYPER, NDRSMALL, NDRPOINTERNULL, NDRSTRUCT, \
+    NDRUSMALL, NDRBOOLEAN, NDRUSHORT, NDRFLOAT, NDRDOUBLEFLOAT, NULL
 
 DWORD = NDRULONG
 BOOL = NDRULONG
 UCHAR = NDRUSMALL
 SHORT = NDRSHORT
+NULL = NULL
+
+class LPDWORD(NDRPOINTER):
+    referent = (
+        ('Data', DWORD),
+    )
 
 class PSHORT(NDRPOINTER):
     referent = (
@@ -55,7 +61,12 @@ class WIDESTR(NDRUniFixedArray):
 
     def __setitem__(self, key, value):
         if key == 'Data':
-            self.fields[key] = value.encode('utf-16le')
+            try:
+                self.fields[key] = value.encode('utf-16le')
+            except UnicodeDecodeError:
+                import sys
+                self.fields[key] = value.decode(sys.getfilesystemencoding()).encode('utf-16le')
+
             self.data = None        # force recompute
         else:
             return NDR.__setitem__(self, key, value)
@@ -83,9 +94,8 @@ class STR(NDRSTRUCT):
 
     def dump(self, msg = None, indent = 0):
         if msg is None: msg = self.__class__.__name__
-        ind = ' '*indent
         if msg != '':
-            print "%s" % (msg),
+            print "%s" % msg,
         # Here just print the data
         print " %r" % (self['Data']),
 
@@ -123,9 +133,8 @@ class WSTR(NDRSTRUCT):
 
     def dump(self, msg = None, indent = 0):
         if msg is None: msg = self.__class__.__name__
-        ind = ' '*indent
         if msg != '':
-            print "%s" % (msg),
+            print "%s" % msg,
         # Here just print the data
         print " %r" % (self['Data']),
 
@@ -134,7 +143,11 @@ class WSTR(NDRSTRUCT):
 
     def __setitem__(self, key, value):
         if key == 'Data':
-            self.fields[key] = value.encode('utf-16le')
+            try:
+                self.fields[key] = value.encode('utf-16le')
+            except UnicodeDecodeError:
+                import sys
+                self.fields[key] = value.decode(sys.getfilesystemencoding()).encode('utf-16le')
             self.fields['MaximumCount'] = None
             self.fields['ActualCount'] = None
             self.data = None        # force recompute
@@ -203,12 +216,24 @@ class PLONGLONG(NDRPOINTER):
         ('Data', LONGLONG),
     )
 
+# 2.2.31 LONG64
+LONG64 = NDRUHYPER
+class PLONG64(NDRPOINTER):
+    referent = (
+        ('Data', LONG64),
+    )
+
 # 2.2.32 LPCSTR
 LPCSTR = LPSTR
 
 # 2.2.36 NET_API_STATUS
 NET_API_STATUS = DWORD
 
+# 2.2.52 ULONG_PTR
+ULONG_PTR = NDRULONG
+# 2.2.10 DWORD_PTR
+DWORD_PTR = ULONG_PTR
+
 # 2.3.2 GUID and UUID
 class GUID(NDRSTRUCT):
     structure = (
@@ -263,6 +288,14 @@ class PUSHORT(NDRPOINTER):
 WCHAR = WSTR
 PWCHAR = LPWSTR
 
+# 2.2.61 WORD
+WORD = NDRUSHORT
+class PWORD(NDRPOINTER):
+    referent = (
+        ('Data', WORD),
+    )
+LPWORD = PWORD
+
 # 2.3.1 FILETIME
 class FILETIME(NDRSTRUCT):
     structure = (
@@ -313,15 +346,19 @@ class RPC_UNICODE_STRING(NDRSTRUCT):
 
     def __setitem__(self, key, value):
         if key == 'Data' and isinstance(value, NDR) is False:
+            try:
+                value.encode('utf-16le')
+            except UnicodeDecodeError:
+                import sys
+                value = value.decode(sys.getfilesystemencoding())
             self['Length'] = len(value)*2
             self['MaximumLength'] = len(value)*2
         return NDRSTRUCT.__setitem__(self, key, value)
 
     def dump(self, msg = None, indent = 0):
         if msg is None: msg = self.__class__.__name__
-        ind = ' '*indent
         if msg != '':
-            print "%s" % (msg),
+            print "%s" % msg,
 
         if isinstance(self.fields['Data'] , NDRPOINTERNULL):
             print " NULL",
@@ -335,15 +372,53 @@ class PRPC_UNICODE_STRING(NDRPOINTER):
        ('Data', RPC_UNICODE_STRING ),
     )
 
+# 2.3.9 OBJECT_TYPE_LIST
+ACCESS_MASK = DWORD
+class OBJECT_TYPE_LIST(NDRSTRUCT):
+    structure = (
+        ('Level', WORD),
+        ('Remaining',ACCESS_MASK),
+        ('ObjectType',PGUID),
+    )
+
+class POBJECT_TYPE_LIST(NDRPOINTER):
+    referent = (
+       ('Data', OBJECT_TYPE_LIST ),
+    )
+
+# 2.3.13 SYSTEMTIME
+class SYSTEMTIME(NDRSTRUCT):
+    structure = (
+        ('wYear', WORD),
+        ('wMonth', WORD),
+        ('wDayOfWeek', WORD),
+        ('wDay', WORD),
+        ('wHour', WORD),
+        ('wMinute', WORD),
+        ('wSecond', WORD),
+        ('wMilliseconds', WORD),
+    )
+
+class PSYSTEMTIME(NDRPOINTER):
+    referent = (
+       ('Data', SYSTEMTIME ),
+    )
+
+# 2.3.15 ULARGE_INTEGER
+class ULARGE_INTEGER(NDRSTRUCT):
+    structure = (
+        ('QuadPart', LONG64),
+    )
+
+class PULARGE_INTEGER(NDRPOINTER):
+    referent = (
+        ('Data', ULARGE_INTEGER),
+    )
+
 # 2.4.2.3 RPC_SID
 class DWORD_ARRAY(NDRUniConformantArray):
     item = '<L'
 
-class LPDWORD(NDRPOINTER):
-    referent = (
-        ('Data', DWORD_ARRAY),
-    )
-
 class RPC_SID_IDENTIFIER_AUTHORITY(NDRUniFixedArray):
     align = 1
     align64 = 1
@@ -367,7 +442,6 @@ class RPC_SID(NDRSTRUCT):
         self['IdentifierAuthority'] = RPC_SID_IDENTIFIER_AUTHORITY()
         self['IdentifierAuthority'] = '\x00\x00\x00\x00\x00' + pack('B',int(items[2]))
         self['SubAuthorityCount'] = len(items) - 3
-        ans = ''
         for i in range(self['SubAuthorityCount']):
             self['SubAuthority'].append(int(items[i+3]))
 
@@ -385,7 +459,6 @@ class PRPC_SID(NDRPOINTER):
 PSID = PRPC_SID
 
 # 2.4.3 ACCESS_MASK
-ACCESS_MASK = DWORD
 GENERIC_READ            = 0x80000000L
 GENERIC_WRITE           = 0x4000000L
 GENERIC_EXECUTE         = 0x20000000L
