@@ -1,57 +1,47 @@
-# Copyright (c) 2003-2014 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: lsad.py 1244 2014-06-26 13:54:47Z bethus@gmail.com $
-#
-# Author: Alberto Solino
+# Author: Alberto Solino (@agsolino)
 #
 # Description:
 #   [MS-LSAD] Interface implementation
 #
 #   Best way to learn how to use these calls is to grab the protocol standard
 #   so you understand what the call does, and then read the test case located
-#   at https://code.google.com/p/impacket/source/browse/#svn%2Ftrunk%2Fimpacket%2Ftestcases%2FSMB_RPC
+#   at https://github.com/CoreSecurity/impacket/tree/master/impacket/testcases/SMB_RPC
 #
 #   Some calls have helper functions, which makes it even easier to use.
 #   They are located at the end of this file. 
 #   Helper functions start with "h"<name of the call>.
 #   There are test cases for them too. 
 #
-from impacket.dcerpc.v5 import ndr
-from impacket.dcerpc.v5.ndr import NDRCALL, NDR, NDRSTRUCT, NDRENUM, NDRUNION, NDRPOINTER, NDRUniConformantArray, NDRUniConformantVaryingArray
-from impacket.dcerpc.v5.dtypes import *
+from impacket.dcerpc.v5.ndr import NDRCALL, NDRENUM, NDRUNION, NDRUniConformantVaryingArray, NDRPOINTER, NDR, NDRSTRUCT, \
+    NDRUniConformantArray
+from impacket.dcerpc.v5.dtypes import DWORD, LPWSTR, STR, LUID, LONG, ULONG, RPC_UNICODE_STRING, PRPC_SID, LPBYTE, \
+    LARGE_INTEGER, NTSTATUS, RPC_SID, ACCESS_MASK, UCHAR, PRPC_UNICODE_STRING, PLARGE_INTEGER, USHORT, \
+    SECURITY_INFORMATION, NULL, MAXIMUM_ALLOWED, GUID, SECURITY_DESCRIPTOR, OWNER_SECURITY_INFORMATION
 from impacket import nt_errors
 from impacket.uuid import uuidtup_to_bin
 from impacket.dcerpc.v5.enum import Enum
+from impacket.dcerpc.v5.rpcrt import DCERPCException
 
 MSRPC_UUID_LSAD  = uuidtup_to_bin(('12345778-1234-ABCD-EF00-0123456789AB','0.0'))
 
-class DCERPCSessionError(Exception):
-    def __init__( self, packet = None, error_code = None):
-        Exception.__init__(self)
-        self.packet = packet
-        if packet is not None:
-            self.error_code = packet['ErrorCode']
-        else:
-            self.error_code = error_code
-       
-    def get_error_code( self ):
-        return self.error_code
- 
-    def get_packet( self ):
-        return self.packet
+class DCERPCSessionError(DCERPCException):
+    def __init__(self, error_string=None, error_code=None, packet=None):
+        DCERPCException.__init__(self, error_string, error_code, packet)
 
     def __str__( self ):
         key = self.error_code
-        if (nt_errors.ERROR_MESSAGES.has_key(key)):
+        if nt_errors.ERROR_MESSAGES.has_key(key):
             error_msg_short = nt_errors.ERROR_MESSAGES[key][0]
             error_msg_verbose = nt_errors.ERROR_MESSAGES[key][1] 
             return 'LSAD SessionError: code: 0x%x - %s - %s' % (self.error_code, error_msg_short, error_msg_verbose)
         else:
-            return 'LSAD SessionError: unknown error code: 0x%x' % (self.error_code)
+            return 'LSAD SessionError: unknown error code: 0x%x' % self.error_code
 
 ################################################################################
 # CONSTANTS
@@ -130,7 +120,7 @@ LSA_FTRECORD_DISABLED_REASONS = 0x0000FFFF
 # STRUCTURES
 ################################################################################
 # 2.2.2.1 LSAPR_HANDLE
-class LSAPR_HANDLE(NDR):
+class LSAPR_HANDLE(NDRSTRUCT):
     align = 1
     structure =  (
         ('Data','20s=""'),
@@ -1212,7 +1202,7 @@ class LsarEnumerateTrustedDomains(NDRCALL):
 class LsarEnumerateTrustedDomainsResponse(NDRCALL):
     structure = (
        ('EnumerationContext', ULONG),
-       ('EnumerationBuffer',LSAPR_TRUSTED_ENUM_BUFFER_EX),
+       ('EnumerationBuffer',LSAPR_TRUSTED_ENUM_BUFFER),
        ('ErrorCode', NTSTATUS),
     )
 
@@ -1591,14 +1581,10 @@ def hLsarSetSecret(dce, secretHandle, encryptedCurrentValue, encryptedOldValue):
         request['EncryptedCurrentValue']['Length'] = len(encryptedCurrentValue)
         request['EncryptedCurrentValue']['MaximumLength'] = len(encryptedCurrentValue)
         request['EncryptedCurrentValue']['Buffer'] = list(encryptedCurrentValue)
-    else:
-        encryptedCurrentValue = NULL
     if encryptedOldValue != NULL:
         request['EncryptedOldValue']['Length'] = len(encryptedOldValue)
         request['EncryptedOldValue']['MaximumLength'] = len(encryptedOldValue)
         request['EncryptedOldValue']['Buffer'] = list(encryptedOldValue)
-    else:
-        encryptedOldValue = NULL
     return dce.request(request)
 
 def hLsarQuerySecret(dce, secretHandle):
