@@ -1,67 +1,56 @@
-# Copyright (c) 2003-2014 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: nrpc.py 1244 2014-06-26 13:54:47Z bethus@gmail.com $
-#
-# Author: Alberto Solino
+# Author: Alberto Solino (@agsolino)
 #
 # Description:
 #   [MS-NRPC] Interface implementation
 #
 #   Best way to learn how to use these calls is to grab the protocol standard
 #   so you understand what the call does, and then read the test case located
-#   at https://code.google.com/p/impacket/source/browse/#svn%2Ftrunk%2Fimpacket%2Ftestcases%2FSMB_RPC
+#   at https://github.com/CoreSecurity/impacket/tree/master/impacket/testcases/SMB_RPC
 #
 #   Some calls have helper functions, which makes it even easier to use.
 #   They are located at the end of this file. 
 #   Helper functions start with "h"<name of the call>.
 #   There are test cases for them too. 
 #
-from impacket.dcerpc.v5 import ndr
-from impacket.dcerpc.v5.ndr import NDRCALL, NDR, NDRSTRUCT, NDRENUM, NDRUNION, NDRPOINTER, NDRUniConformantArray, NDRUniFixedArray, NDRUniConformantVaryingArray
-from impacket.dcerpc.v5.dtypes import *
+from struct import pack
+from impacket.dcerpc.v5.ndr import NDRCALL, NDRSTRUCT, NDRENUM, NDRUNION, NDRPOINTER, NDRUniConformantArray, \
+    NDRUniFixedArray, NDRUniConformantVaryingArray
+from impacket.dcerpc.v5.dtypes import WSTR, LPWSTR, DWORD, ULONG, USHORT, PGUID, NTSTATUS, NULL, LONG, UCHAR, PRPC_SID, \
+    GUID, RPC_UNICODE_STRING, SECURITY_INFORMATION, LPULONG
 from impacket import system_errors, nt_errors
 from impacket.uuid import uuidtup_to_bin
 from impacket.dcerpc.v5.enum import Enum
 from impacket.dcerpc.v5.samr import OLD_LARGE_INTEGER
 from impacket.dcerpc.v5.lsad import PLSA_FOREST_TRUST_INFORMATION
+from impacket.dcerpc.v5.rpcrt import DCERPCException
 from impacket.structure import Structure
-from impacket import ntlm
-from impacket import crypto
+from impacket import ntlm, crypto, LOG
 import hmac, hashlib
 try:
- from Crypto.Cipher import DES, AES, ARC4
+    from Crypto.Cipher import DES, AES, ARC4
 except Exception:
-    print "Warning: You don't have any crypto installed. You need PyCrypto"
-    print "See http://www.pycrypto.org/"
+    LOG.critical("Warning: You don't have any crypto installed. You need PyCrypto")
+    LOG.critical("See http://www.pycrypto.org/")
 
 MSRPC_UUID_NRPC = uuidtup_to_bin(('12345678-1234-ABCD-EF00-01234567CFFB', '1.0'))
 
-class DCERPCSessionError(Exception):
-    def __init__( self, packet = None, error_code = None):
-        Exception.__init__(self)
-        self.packet = packet
-        if packet is not None:
-            self.error_code = packet['ErrorCode']
-        else:
-            self.error_code = error_code
-       
-    def get_error_code( self ):
-        return self.error_code
- 
-    def get_packet( self ):
-        return self.packet
+class DCERPCSessionError(DCERPCException):
+    def __init__(self, error_string=None, error_code=None, packet=None):
+        DCERPCException.__init__(self, error_string, error_code, packet)
 
     def __str__( self ):
         key = self.error_code
-        if (system_errors.ERROR_MESSAGES.has_key(key)):
+        if system_errors.ERROR_MESSAGES.has_key(key):
             error_msg_short = system_errors.ERROR_MESSAGES[key][0]
             error_msg_verbose = system_errors.ERROR_MESSAGES[key][1] 
             return 'NRPC SessionError: code: 0x%x - %s - %s' % (self.error_code, error_msg_short, error_msg_verbose)
-        elif (nt_errors.ERROR_MESSAGES.has_key(key)):
+        elif nt_errors.ERROR_MESSAGES.has_key(key):
             error_msg_short = nt_errors.ERROR_MESSAGES[key][0]
             error_msg_verbose = nt_errors.ERROR_MESSAGES[key][1] 
             return 'NRPC SessionError: code: 0x%x - %s - %s' % (self.error_code, error_msg_short, error_msg_verbose)
@@ -239,11 +228,6 @@ class PNL_SITE_NAME_ARRAY(NDRPOINTER):
 class RPC_UNICODE_STRING_ARRAY(NDRUniConformantArray):
     item = RPC_UNICODE_STRING
 
-class PRPC_UNICODE_STRING_ARRAY(NDRPOINTER):
-    referent = (
-        ('Data', RPC_UNICODE_STRING_ARRAY),
-    )
-
 class NL_SITE_NAME_EX_ARRAY(NDRSTRUCT):
     structure = (
         ('EntryCount', ULONG),
@@ -1679,8 +1663,6 @@ def ComputeSessionKeyStrongKey(sharedSecret, clientChallenge, serverChallenge, s
     return hm.digest()
 
 def deriveSequenceNumber(sequenceNum):
-    res = ''
-
     sequenceLow = sequenceNum & 0xffffffff
     sequenceHigh = (sequenceNum >> 32) & 0xffffffff
     sequenceHigh |= 0x80000000
@@ -1833,13 +1815,13 @@ def getSSPType1(workstation='', domain='', signingRequired=False):
     if domain != '':
         auth['Buffer'] = auth['Buffer'] + domain + '\x00'
     else:
-        auth['Buffer'] = auth['Buffer'] + 'WORKGROUP\x00'
+        auth['Buffer'] += 'WORKGROUP\x00'
 
     auth['Flags'] |= NL_AUTH_MESSAGE_NETBIOS_HOST 
     if workstation != '':
         auth['Buffer'] = auth['Buffer'] + workstation + '\x00'
     else:
-        auth['Buffer'] = auth['Buffer'] + 'MYHOST\x00'
+        auth['Buffer'] += 'MYHOST\x00'
 
     auth['Flags'] |= NL_AUTH_MESSAGE_NETBIOS_HOST_UTF8 
     if workstation != '':
