@@ -1,58 +1,46 @@
-# Copyright (c) 2003-2014 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: rrp.py 1244 2014-06-26 13:54:47Z bethus@gmail.com $
-#
-# Author: Alberto Solino
+# Author: Alberto Solino (@agsolino)
 #
 # Description:
 #   [MS-RRP] Interface implementation
 #
 #   Best way to learn how to use these calls is to grab the protocol standard
 #   so you understand what the call does, and then read the test case located
-#   at https://code.google.com/p/impacket/source/browse/#svn%2Ftrunk%2Fimpacket%2Ftestcases%2FSMB_RPC
+#   at https://github.com/CoreSecurity/impacket/tree/master/impacket/testcases/SMB_RPC
 #
 #   Some calls have helper functions, which makes it even easier to use.
 #   They are located at the end of this file. 
 #   Helper functions start with "h"<name of the call>.
 #   There are test cases for them too. 
 #
+from struct import unpack, pack
 
-from impacket.dcerpc.v5 import ndr
-from impacket.dcerpc.v5.ndr import NDRCALL, NDR, NDRSTRUCT, NDRPOINTER, NDRUniConformantVaryingArray
-from impacket.dcerpc.v5.dtypes import *
+from impacket.dcerpc.v5.ndr import NDRCALL, NDRSTRUCT, NDRPOINTER, NDRUniConformantVaryingArray, NDRUniConformantArray
+from impacket.dcerpc.v5.dtypes import DWORD, UUID, ULONG, LPULONG, BOOLEAN, SECURITY_INFORMATION, PFILETIME, \
+    RPC_UNICODE_STRING, FILETIME, NULL, MAXIMUM_ALLOWED, OWNER_SECURITY_INFORMATION, PWCHAR, PRPC_UNICODE_STRING
+from impacket.dcerpc.v5.rpcrt import DCERPCException
 from impacket import system_errors
 from impacket.uuid import uuidtup_to_bin
-from impacket.dcerpc.v5.enum import Enum
 
 MSRPC_UUID_RRP = uuidtup_to_bin(('338CD001-2244-31F1-AAAA-900038001003', '1.0'))
 
-class DCERPCSessionError(Exception):
-    def __init__( self, packet = None, error_code = None):
-        Exception.__init__(self)
-        self.packet = packet
-        if packet is not None:
-            self.error_code = packet['ErrorCode']
-        else:
-            self.error_code = error_code
-       
-    def get_error_code( self ):
-        return self.error_code
- 
-    def get_packet( self ):
-        return self.packet
+class DCERPCSessionError(DCERPCException):
+    def __init__(self, error_string=None, error_code=None, packet=None):
+        DCERPCException.__init__(self, error_string, error_code, packet)
 
     def __str__( self ):
         key = self.error_code
-        if (system_errors.ERROR_MESSAGES.has_key(key)):
+        if system_errors.ERROR_MESSAGES.has_key(key):
             error_msg_short = system_errors.ERROR_MESSAGES[key][0]
             error_msg_verbose = system_errors.ERROR_MESSAGES[key][1] 
             return 'RRP SessionError: code: 0x%x - %s - %s' % (self.error_code, error_msg_short, error_msg_verbose)
         else:
-            return 'RRP SessionError: unknown error code: 0x%x' % (self.error_code)
+            return 'RRP SessionError: unknown error code: 0x%x' % self.error_code
 
 ################################################################################
 # CONSTANTS
@@ -687,6 +675,38 @@ def checkNullString(string):
     else:
         return string
 
+def packValue(valueType, value):
+    if valueType == REG_DWORD:
+        retData = pack('<L', value)
+    elif valueType == REG_DWORD_BIG_ENDIAN:
+        retData = pack('>L', value)
+    elif valueType == REG_EXPAND_SZ:
+        try:
+            retData = value.encode('utf-16le')
+        except UnicodeDecodeError:
+            import sys
+            retData = value.decode(sys.getfilesystemencoding()).encode('utf-16le')
+    elif valueType == REG_MULTI_SZ:
+        try:
+            retData = value.encode('utf-16le')
+        except UnicodeDecodeError:
+            import sys
+            retData = value.decode(sys.getfilesystemencoding()).encode('utf-16le')
+    elif valueType == REG_QWORD:
+        retData = pack('<Q', value)
+    elif valueType == REG_QWORD_LITTLE_ENDIAN:
+        retData = pack('>Q', value)
+    elif valueType == REG_SZ:
+        try:
+            retData = value.encode('utf-16le')
+        except UnicodeDecodeError:
+            import sys
+            retData = value.decode(sys.getfilesystemencoding()).encode('utf-16le')
+    else:
+        retData = value
+
+    return retData
+
 def unpackValue(valueType, value):
     if valueType == REG_DWORD:
         retData = unpack('<L', ''.join(value))[0]
@@ -774,18 +794,30 @@ def hBaseRegEnumKey(dce, hKey, dwIndex, lpftLastWriteTime = NULL):
 
     return dce.request(request)
 
-def hBaseRegEnumValue(dce, hKey, dwIndex):
-    # ToDo, check the result to see whether we need to 
-    # have a bigger buffer for the data to receive
+def hBaseRegEnumValue(dce, hKey, dwIndex, dataLen=128):
     request = BaseRegEnumValue()
     request['hKey'] = hKey
     request['dwIndex'] = dwIndex
-    request['lpValueNameIn'] = ' '*128
-    request['lpData'] = ' '*128
-    request['lpcbData'] = 128
-    request['lpcbLen'] = 128
 
-    return dce.request(request)
+    # We need to be aware the size might not be enough, so let's catch ERROR_MORE_DATA exception
+    while True:
+        try:
+            request['lpValueNameIn'] = ' ' * dataLen
+            request['lpData'] = ' ' * dataLen
+            request['lpcbData'] = dataLen
+            request['lpcbLen'] = dataLen
+            resp = dce.request(request)
+        except DCERPCSessionError, e:
+            if e.get_error_code() == system_errors.ERROR_MORE_DATA:
+                # We need to adjust the size
+                dataLen = e.get_packet()['lpcbData']
+                continue
+            else:
+                raise
+        else:
+            break
+
+    return resp
 
 def hBaseRegFlushKey(dce, hKey):
     request = BaseRegFlushKey()
@@ -831,17 +863,29 @@ def hBaseRegQueryInfoKey(dce, hKey):
     request.fields['lpClassIn'].fields['Data'].fields['Data'].fields['MaximumCount'] = 1024/2
     return dce.request(request)
 
-def hBaseRegQueryValue(dce, hKey, lpValueName):
-    # ToDo, check the result to see whether we need to 
-    # have a bigger buffer for the data to receive
+def hBaseRegQueryValue(dce, hKey, lpValueName, dataLen=512):
     request = BaseRegQueryValue()
     request['hKey'] = hKey
     request['lpValueName'] = checkNullString(lpValueName)
-    request['lpData'] = ' '*512
-    request['lpcbData'] = 512
-    request['lpcbLen'] = 512
-    resp = dce.request(request)
-    # Returns 
+
+    # We need to be aware the size might not be enough, so let's catch ERROR_MORE_DATA exception
+    while True:
+        try:
+            request['lpData'] = ' ' * dataLen
+            request['lpcbData'] = dataLen
+            request['lpcbLen'] = dataLen
+            resp = dce.request(request)
+        except DCERPCSessionError, e:
+            if e.get_error_code() == system_errors.ERROR_MORE_DATA:
+                # We need to adjust the size
+                dataLen = e.get_packet()['lpcbData']
+                continue
+            else:
+                raise
+        else:
+            break
+
+    # Returns
     # ( dataType, data )
     return resp['lpType'], unpackValue(resp['lpType'], resp['lpData'])
 
@@ -872,7 +916,7 @@ def hBaseRegSetValue(dce, hKey, lpValueName, dwType, lpData):
     request['hKey'] = hKey
     request['lpValueName'] = checkNullString(lpValueName)
     request['dwType'] = dwType
-    request['lpData'] = lpData.encode('utf-16le')
+    request['lpData'] = packValue(dwType,lpData)
     request['cbData'] = len(request['lpData'])
     return dce.request(request)
 
@@ -908,7 +952,7 @@ def hBaseRegQueryMultipleValues(dce, hKey, val_listIn):
     resp = dce.request(request)
     retVal = list()
     for item in resp['val_listOut']:
-        itemn = {}
+        itemn = dict()
         itemn['ValueName'] = item['ve_valuename'] 
         itemn['ValueData'] = unpackValue(item['ve_type'], resp['lpvalueBuf'][item['ve_valueptr'] : item['ve_valueptr']+item['ve_valuelen']])
         retVal.append(itemn)
