@@ -1,57 +1,45 @@
-# Copyright (c) 2003-2014 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: srvs.py 1244 2014-06-26 13:54:47Z bethus@gmail.com $
-#
-# Author: Alberto Solino
+# Author: Alberto Solino (@agsolino)
 #
 # Description:
 #   [MS-SRVS] Interface implementation
 #
 #   Best way to learn how to use these calls is to grab the protocol standard
 #   so you understand what the call does, and then read the test case located
-#   at https://code.google.com/p/impacket/source/browse/#svn%2Ftrunk%2Fimpacket%2Ftestcases%2FSMB_RPC
+#   at https://github.com/CoreSecurity/impacket/tree/master/impacket/testcases/SMB_RPC
 #
 #   Some calls have helper functions, which makes it even easier to use.
 #   They are located at the end of this file. 
 #   Helper functions start with "h"<name of the call>.
 #   There are test cases for them too. 
 #
-from impacket.dcerpc.v5 import ndr
-from impacket.dcerpc.v5.ndr import NDRCALL, NDR, NDRSTRUCT, NDRENUM, NDRUNION, NDRPOINTER, NDRUniConformantArray, NDRUniFixedArray, NDRBOOLEAN, NDRUniConformantVaryingArray, NDRUniVaryingArray, PNDRUniConformantArray
-from impacket.dcerpc.v5.dtypes import *
+from impacket.dcerpc.v5.rpcrt import DCERPCException
+from impacket.dcerpc.v5.ndr import NDRCALL, NDR, NDRSTRUCT, NDRUNION, NDRPOINTER, NDRUniConformantArray, \
+    NDRUniFixedArray, NDRBOOLEAN, NDRUniConformantVaryingArray, PNDRUniConformantArray
+from impacket.dcerpc.v5.dtypes import NULL, DWORD, LPWSTR, LPBYTE, LMSTR, ULONG, GUID, LPLONG, WSTR, \
+    SECURITY_INFORMATION, WCHAR
 from impacket import system_errors
 from impacket.uuid import uuidtup_to_bin
-from impacket.dcerpc.v5.enum import Enum
 
 MSRPC_UUID_SRVS  = uuidtup_to_bin(('4B324FC8-1670-01D3-1278-5A47BF6EE188', '3.0'))
 
-class DCERPCSessionError(Exception):
-    def __init__( self, packet = None, error_code = None):
-        Exception.__init__(self)
-        self.packet = packet
-        if packet is not None:
-            self.error_code = packet['ErrorCode']
-        else:
-            self.error_code = error_code
-       
-    def get_error_code( self ):
-        return self.error_code
- 
-    def get_packet( self ):
-        return self.packet
+class DCERPCSessionError(DCERPCException):
+    def __init__(self, error_string=None, error_code=None, packet=None):
+        DCERPCException.__init__(self, error_string, error_code, packet)
 
     def __str__( self ):
         key = self.error_code
-        if (system_errors.ERROR_MESSAGES.has_key(key)):
+        if system_errors.ERROR_MESSAGES.has_key(key):
             error_msg_short = system_errors.ERROR_MESSAGES[key][0]
             error_msg_verbose = system_errors.ERROR_MESSAGES[key][1] 
             return 'SRVS SessionError: code: 0x%x - %s - %s' % (self.error_code, error_msg_short, error_msg_verbose)
         else:
-            return 'SRVS SessionError: unknown error code: 0x%x' % (self.error_code)
+            return 'SRVS SessionError: unknown error code: 0x%x' % self.error_code
 
 ################################################################################
 # CONSTANTS
@@ -65,7 +53,7 @@ class PSRVSVC_HANDLE(NDRPOINTER):
     )
 
 # 2.2.1.2 SHARE_DEL_HANDLE
-class SHARE_DEL_HANDLE(NDR):
+class SHARE_DEL_HANDLE(NDRSTRUCT):
     align = 1
     structure =  (
         ('Data','20s=""'),
@@ -1764,15 +1752,18 @@ class WCHAR_ARRAY(NDRSTRUCT):
 
     def dump(self, msg = None, indent = 0):
         if msg is None: msg = self.__class__.__name__
-        ind = ' '*indent
         if msg != '':
-            print "%s" % (msg),
+            print "%s" % msg,
         # Here just print the data
         print " %r" % (self['Data']),
 
     def __setitem__(self, key, value):
         if key == 'Data':
-            self.fields[key] = value.encode('utf-16le')
+            try:
+                self.fields[key] = value.encode('utf-16le')
+            except UnicodeDecodeError:
+                import sys
+                self.fields[key] = value.decode(sys.getfilesystemencoding()).encode('utf-16le')
             self.fields['ActualCount'] = None
             self.data = None        # force recompute
         else:
@@ -3097,7 +3088,7 @@ def hNetrShareDel(dce, netName):
 
 def hNetrShareEnum(dce, level, resumeHandle = 0, preferedMaximumLength = 0xffffffff):
     request = NetrShareEnum()
-    request['ServerName'] = NULL
+    request['ServerName'] = '\x00'
     request['PreferedMaximumLength'] = preferedMaximumLength
     request['ResumeHandle'] = resumeHandle
     request['InfoStruct']['Level'] = level
