@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2013 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -10,7 +10,7 @@
 #             to parse NTDS.dit files (not meant as a full parser, although it might work)
 #
 # Author:
-#  Alberto Solino
+#  Alberto Solino (@agsolino)
 #
 # Reference for:
 #  Structure.
@@ -30,9 +30,9 @@ except:
         from ordereddict.ordereddict import OrderedDict
     except:
         from ordereddict import OrderedDict
-from impacket import structure, version
 from impacket.structure import Structure
 from struct import unpack
+from binascii import hexlify
 
 # Constants
 
@@ -272,7 +272,7 @@ class ESENT_PAGE_HEADER(Structure):
         if (version < 0x620) or (version == 0x620 and revision < 0x0b):
             # For sure the old format
             self.structure = self.structure_2003_SP0 + self.common
-        elif (version == 0x620 and revision < 0x11):
+        elif version == 0x620 and revision < 0x11:
             # Exchange 2003 SP1 and Windows Vista and later
             self.structure = self.structure_0x620_0x0b + self.common
         else:
@@ -281,7 +281,7 @@ class ESENT_PAGE_HEADER(Structure):
             if pageSize > 8192:
                 self.structure += self.extended_win7
 
-        return Structure.__init__(self,data)
+        Structure.__init__(self,data)
 
 class ESENT_ROOT_HEADER(Structure):
     structure = (
@@ -310,7 +310,7 @@ class ESENT_BRANCH_ENTRY(Structure):
         if flags & TAG_COMMON > 0:
             # Include the common header
             self.structure = self.common + self.structure
-        return Structure.__init__(self,data)
+        Structure.__init__(self,data)
 
 class ESENT_LEAF_HEADER(Structure):
     structure = (
@@ -331,7 +331,7 @@ class ESENT_LEAF_ENTRY(Structure):
         if flags & TAG_COMMON > 0:
             # Include the common header
             self.structure = self.common + self.structure
-        return Structure.__init__(self,data)
+        Structure.__init__(self,data)
 
 class ESENT_SPACE_TREE_HEADER(Structure):
     structure = (
@@ -419,13 +419,13 @@ class ESENT_CATALOG_DATA_DEFINITION_ENTRY(Structure):
         else:
             LOG.error('Unknown catalog type 0x%x' % dataType)
             self.structure = ()
-            return Structure.__init__(self,data)
+            Structure.__init__(self,data)
 
         self.structure += self.common
 
-        return Structure.__init__(self,data)
+        Structure.__init__(self,data)
+
 
-import string
 def pretty_print(x):
     if x in '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~ ':
        return x
@@ -455,7 +455,7 @@ def getUnixTime(t):
     t /= 10000000
     return t
 
-class ESENT_PAGE():
+class ESENT_PAGE:
     def __init__(self, db, data=None):
         self.__DBHeader = db
         self.data = data
@@ -625,7 +625,7 @@ class ESENT_DB:
         indent = '    '
 
         print "Database version: 0x%x, 0x%x" % (self.__DBHeader['Version'], self.__DBHeader['FileFormatRevision'] )
-        print "Page size: %d " % (self.__pageSize)
+        print "Page size: %d " % self.__pageSize
         print "Number of pages: %d" % self.__totalPages
         print 
         print "Catalog for %s" % self.__fileName
@@ -799,7 +799,7 @@ class ESENT_DB:
         tag = self.__getNextTag(cursor)
         #hexdump(tag)
 
-        if tag == None:
+        if tag is None:
             # No more tags in this page, search for the next one on the right
             page = cursor['CurrentPageData']
             if page.record['NextPageNumber'] == 0:
@@ -862,7 +862,7 @@ class ESENT_DB:
                 record[column] = tag[fixedSizeOffset:][:columnRecord['SpaceUsage']]
                 fixedSizeOffset += columnRecord['SpaceUsage']
 
-            elif columnRecord['Identifier'] > 127 and columnRecord['Identifier'] <= dataDefinitionHeader['LastVariableDataType']:
+            elif 127 < columnRecord['Identifier'] <= dataDefinitionHeader['LastVariableDataType']:
                 # Variable data type
                 index = columnRecord['Identifier'] - 127 - 1
                 itemLen = unpack('<H',tag[variableSizeOffset+index*2:][:2])[0]
@@ -936,7 +936,7 @@ class ESENT_DB:
                     elif itemFlag & TAGGED_DATA_TYPE_MULTI_VALUE:
                         # ToDo: Parse multi-values properly
                         LOG.debug('Multivalue detected in column %s, returning raw results' % (column))
-                        record[column] = (tag[offsetItem:][:itemSize].encode('hex'),)
+                        record[column] = (hexlify(tag[offsetItem:][:itemSize]),)
                     else:
                         record[column] = tag[offsetItem:][:itemSize]
 
@@ -964,7 +964,7 @@ class ESENT_DB:
                 unpackData = ColumnTypeSize[columnRecord['ColumnType']]
                 if record[column] is not None:
                     if unpackData is None:
-                        record[column] = record[column].encode('hex')
+                        record[column] = hexlify(record[column])
                     else:
                         unpackStr = unpackData[1]
                         unpackSize = unpackData[0]
