@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2012 CORE Security Technologies
+# Copyright (c) 2003-2016 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -15,28 +15,29 @@
 #     rewritten to output like a normal SQL client
 #
 # Author:
-#  Alberto Solino (beto@coresecurity.com)
+#  Alberto Solino (@agsolino)
 #
 
 
-from impacket import ntlm, uuid, LOG
-from impacket.structure import Structure
-import random
-import string
 import struct
-import socket, select
+import socket
+import select
 import random
-import binascii 
+import binascii
 import math
 import datetime
 import sys
+import string
+
+from impacket import ntlm, uuid, LOG
+from impacket.structure import Structure
 
 try:
     import OpenSSL
     from OpenSSL import SSL, crypto
 except:
     LOG.critical("pyOpenSSL is not installed, can't continue")
-    sys.exit(1)
+    raise
 
 # We need to have a fake Logger to be compatible with the way Impact 
 # prints information. Outside Impact it's just a print. Inside 
@@ -422,7 +423,7 @@ class TDS_COLMETADATA(Structure):
         ('Data',':'),
     )
 
-class MSSQL():
+class MSSQL:
     def __init__(self, address, port=1433, rowsPrinter=DummyPrint()):
         #self.packetSize = 32764
         self.packetSize = 32763
@@ -469,9 +470,9 @@ class MSSQL():
         for i, entry in enumerate(entries):
             fields = entry.split(';')
             ret = {}
-            for i, field in enumerate(fields):
-                if (i & 0x1) == 0:
-                    ret[field] = fields[i+1]
+            for j, field in enumerate(fields):
+                if (j & 0x1) == 0:
+                    ret[field] = fields[j+1]
             resp.append(ret)
 
         return resp
@@ -594,8 +595,7 @@ class MSSQL():
         while status != TDS_STATUS_EOM:
             if remaining is not None:
                 tmpPacket = TDSPacket(remaining)
-                remaining = None
-            else: 
+            else:
                 tmpPacket = TDSPacket(self.socketRecv(packetSize))
 
             packetLen = tmpPacket['Length'] - 8
@@ -615,6 +615,225 @@ class MSSQL():
         #print packet['Length']
         return packet
 
+    def kerberosLogin(self, database, username, password='', domain='', hashes=None, aesKey='', kdcHost=None, TGT=None, TGS=None, useCache=True):
+
+        if hashes is not None:
+            lmhash, nthash = hashes.split(':')
+            lmhash = binascii.a2b_hex(lmhash)
+            nthash = binascii.a2b_hex(nthash)
+        else:
+            lmhash = ''
+            nthash = ''
+
+        resp = self.preLogin()
+        # Test this!
+        if resp['Encryption'] == TDS_ENCRYPT_REQ or resp['Encryption'] == TDS_ENCRYPT_OFF:
+            LOG.info("Encryption required, switching to TLS")
+
+            # Switching to TLS now
+            ctx = SSL.Context(SSL.TLSv1_METHOD)
+            ctx.set_cipher_list('RC4')
+            tls = SSL.Connection(ctx,None)
+            tls.set_connect_state()
+            while True:
+                try:
+                    tls.do_handshake()
+                except SSL.WantReadError:
+                    data = tls.bio_read(4096)
+                    self.sendTDS(TDS_PRE_LOGIN, data,0)
+                    tds = self.recvTDS()
+                    tls.bio_write(tds['Data'])
+                else:
+                    break
+
+            # SSL and TLS limitation: Secure Socket Layer (SSL) and its replacement,
+            # Transport Layer Security(TLS), limit data fragments to 16k in size.
+            self.packetSize = 16*1024-1
+            self.tlsSocket = tls
+
+
+        login = TDS_LOGIN()
+
+        login['HostName'] = (''.join([random.choice(string.letters) for _ in range(8)])).encode('utf-16le')
+        login['AppName']  = (''.join([random.choice(string.letters) for _ in range(8)])).encode('utf-16le')
+        login['ServerName'] = self.server.encode('utf-16le')
+        login['CltIntName']  = login['AppName']
+        login['ClientPID'] = random.randint(0,1024)
+        login['PacketSize'] = self.packetSize
+        if database is not None:
+            login['Database'] = database.encode('utf-16le')
+        login['OptionFlags2'] = TDS_INIT_LANG_FATAL | TDS_ODBC_ON
+
+        from impacket.spnego import SPNEGO_NegTokenInit, TypesMech
+        # Importing down here so pyasn1 is not required if kerberos is not used.
+        from impacket.krb5.asn1 import AP_REQ, Authenticator, TGS_REP, seq_set
+        from impacket.krb5.kerberosv5 import getKerberosTGT, getKerberosTGS, KerberosError
+        from impacket.krb5 import constants
+        from impacket.krb5.types import Principal, KerberosTime, Ticket
+        from pyasn1.codec.der import decoder, encoder
+        from impacket.krb5.ccache import CCache
+        import os
+        import datetime
+
+        if useCache is True:
+            try:
+                ccache = CCache.loadFile(os.getenv('KRB5CCNAME'))
+            except:
+                # No cache present
+                pass
+            else:
+                # retrieve user and domain information from CCache file if needed
+                if user == '' and len(ccache.principal.components) > 0:
+                    user=ccache.principal.components[0]['data']
+                if domain == '':
+                    domain = ccache.principal.realm['data']
+                LOG.debug("Using Kerberos Cache: %s" % os.getenv('KRB5CCNAME'))
+                principal = 'MSSQLSvc/%s.%s:%d' % (self.server, domain, self.port)
+                creds = ccache.getCredential(principal)
+                if creds is None:
+                    # Let's try for the TGT and go from there
+                    principal = 'krbtgt/%s@%s' % (domain.upper(),domain.upper())
+                    creds =  ccache.getCredential(principal)
+                    if creds is not None:
+                        TGT = creds.toTGT()
+                        LOG.debug('Using TGT from cache')
+                    else:
+                        LOG.debug("No valid credentials found in cache. ")
+                else:
+                    TGS = creds.toTGS()
+                    LOG.debug('Using TGS from cache')
+
+        # First of all, we need to get a TGT for the user
+        userName = Principal(username, type=constants.PrincipalNameType.NT_PRINCIPAL.value)
+        while True:
+            if TGT is None:
+                if TGS is None:
+                    try:
+                        tgt, cipher, oldSessionKey, sessionKey = getKerberosTGT(userName, password, domain, lmhash, nthash, aesKey, kdcHost)
+                    except KerberosError, e:
+                        if e.getErrorCode() == constants.ErrorCodes.KDC_ERR_ETYPE_NOSUPP.value:
+                            # We might face this if the target does not support AES
+                            # So, if that's the case we'll force using RC4 by converting
+                            # the password to lm/nt hashes and hope for the best. If that's already
+                            # done, byebye.
+                            if lmhash is '' and nthash is '' and (aesKey is '' or aesKey is None) and TGT is None and TGS is None:
+                                from impacket.ntlm import compute_lmhash, compute_nthash
+                                LOG.debug('Got KDC_ERR_ETYPE_NOSUPP, fallback to RC4')
+                                lmhash = compute_lmhash(password)
+                                nthash = compute_nthash(password)
+                                continue
+                            else:
+                                raise
+                        else:
+                            raise
+            else:
+                tgt = TGT['KDC_REP']
+                cipher = TGT['cipher']
+                sessionKey = TGT['sessionKey']
+
+            if TGS is None:
+                # From https://msdn.microsoft.com/en-us/library/ms191153.aspx?f=255&MSPPError=-2147217396
+                # Beginning with SQL Server 2008, the SPN format is changed in order to support Kerberos authentication
+                # on TCP/IP, named pipes, and shared memory. The supported SPN formats for named and default instances
+                # are as follows.
+                # Named instance
+                #     MSSQLSvc/FQDN:[port | instancename], where:
+                #         MSSQLSvc is the service that is being registered.
+                #         FQDN is the fully qualified domain name of the server.
+                #         port is the TCP port number.
+                #         instancename is the name of the SQL Server instance.
+                serverName = Principal('MSSQLSvc/%s.%s:%d' % (self.server, domain, self.port), type=constants.PrincipalNameType.NT_SRV_INST.value)
+                try:
+                    tgs, cipher, oldSessionKey, sessionKey = getKerberosTGS(serverName, domain, kdcHost, tgt, cipher, sessionKey)
+                except KerberosError, e:
+                    if e.getErrorCode() == constants.ErrorCodes.KDC_ERR_ETYPE_NOSUPP.value:
+                        # We might face this if the target does not support AES
+                        # So, if that's the case we'll force using RC4 by converting
+                        # the password to lm/nt hashes and hope for the best. If that's already
+                        # done, byebye.
+                        if lmhash is '' and nthash is '' and (aesKey is '' or aesKey is None) and TGT is None and TGS is None:
+                            from impacket.ntlm import compute_lmhash, compute_nthash
+                            LOG.debug('Got KDC_ERR_ETYPE_NOSUPP, fallback to RC4')
+                            lmhash = compute_lmhash(password)
+                            nthash = compute_nthash(password)
+                        else:
+                            raise
+                    else:
+                        raise
+                else:
+                    break
+            else:
+                tgs = TGS['KDC_REP']
+                cipher = TGS['cipher']
+                sessionKey = TGS['sessionKey']
+                break
+
+        # Let's build a NegTokenInit with a Kerberos REQ_AP
+
+        blob = SPNEGO_NegTokenInit()
+
+        # Kerberos
+        blob['MechTypes'] = [TypesMech['MS KRB5 - Microsoft Kerberos 5']]
+
+        # Let's extract the ticket from the TGS
+        tgs = decoder.decode(tgs, asn1Spec = TGS_REP())[0]
+        ticket = Ticket()
+        ticket.from_asn1(tgs['ticket'])
+
+        # Now let's build the AP_REQ
+        apReq = AP_REQ()
+        apReq['pvno'] = 5
+        apReq['msg-type'] = int(constants.ApplicationTagNumbers.AP_REQ.value)
+
+        opts = list()
+        apReq['ap-options'] = constants.encodeFlags(opts)
+        seq_set(apReq,'ticket', ticket.to_asn1)
+
+        authenticator = Authenticator()
+        authenticator['authenticator-vno'] = 5
+        authenticator['crealm'] = domain
+        seq_set(authenticator, 'cname', userName.components_to_asn1)
+        now = datetime.datetime.utcnow()
+
+        authenticator['cusec'] = now.microsecond
+        authenticator['ctime'] = KerberosTime.to_asn1(now)
+
+        encodedAuthenticator = encoder.encode(authenticator)
+
+        # Key Usage 11
+        # AP-REQ Authenticator (includes application authenticator
+        # subkey), encrypted with the application session key
+        # (Section 5.5.1)
+        encryptedEncodedAuthenticator = cipher.encrypt(sessionKey, 11, encodedAuthenticator, None)
+
+        apReq['authenticator'] = None
+        apReq['authenticator']['etype'] = cipher.enctype
+        apReq['authenticator']['cipher'] = encryptedEncodedAuthenticator
+
+        blob['MechToken'] = encoder.encode(apReq)
+
+        login['OptionFlags2'] |= TDS_INTEGRATED_SECURITY_ON
+
+        login['SSPI'] = blob.getData()
+        login['Length'] = len(str(login))
+
+        # Send the NTLMSSP Negotiate or SQL Auth Packet
+        self.sendTDS(TDS_LOGIN7, str(login))
+
+        # According to the spects, if encryption is not required, we must encrypt just
+        # the first Login packet :-o
+        if resp['Encryption'] == TDS_ENCRYPT_OFF:
+            self.tlsSocket = None
+
+        tds = self.recvTDS()
+
+        self.replies = self.parseReply(tds['Data'])
+
+        if self.replies.has_key(TDS_LOGINACK_TOKEN):
+            return True
+        else:
+            return False
+
     def login(self, database, username, password='', domain='', hashes = None, useWindowsAuth = False):
 
         if hashes is not None:
@@ -667,7 +886,7 @@ class MSSQL():
         if useWindowsAuth is True:
             login['OptionFlags2'] |= TDS_INTEGRATED_SECURITY_ON
             # NTLMSSP Negotiate
-            auth = ntlm.getNTLMSSPType1('WORKSTATION','')
+            auth = ntlm.getNTLMSSPType1('','')
             login['SSPI'] = str(auth)
         else:
             login['UserName'] = username.encode('utf-16le')
@@ -819,7 +1038,7 @@ class MSSQL():
                 else:
                     value = 'NULL'
 
-            elif (_type == TDS_BIGVARCHRTYPE): 
+            elif _type == TDS_BIGVARCHRTYPE:
                 charLen = struct.unpack('<H',data[:struct.calcsize('<H')])[0]
                 data = data[struct.calcsize('<H'):]
                 if charLen != 0xFFFF:
@@ -828,7 +1047,7 @@ class MSSQL():
                 else:
                     value = 'NULL'
 
-            elif (_type == TDS_GUIDTYPE):
+            elif _type == TDS_GUIDTYPE:
                 uuidLen = ord(data[0])
                 data = data[1:]
                 if uuidLen > 0:
@@ -858,7 +1077,7 @@ class MSSQL():
                     else:
                         value = 'NULL'
                 
-            elif (_type == TDS_TEXTTYPE): 
+            elif _type == TDS_TEXTTYPE:
                 # Skip the pointer data
                 charLen = ord(data[0])
                 if charLen == 0:
@@ -898,7 +1117,7 @@ class MSSQL():
                     else:
                         value = 'NULL'
                     data = data[1:]
-                if (_type == TDS_DATETIMETYPE):
+                if _type == TDS_DATETIMETYPE:
                     # datetime is represented in the following sequence:
                     # * One 4-byte signed integer that represents the number of days since January 1, 1900. Negative
                     #   numbers are allowed to represents dates since January 1, 1753.
@@ -912,7 +1131,7 @@ class MSSQL():
                         baseDate = datetime.date(1900,1,1)
                     timeValue = struct.unpack('<L',data[:4])[0]
                     data = data[4:] 
-                elif (_type == TDS_DATETIM4TYPE):
+                elif _type == TDS_DATETIM4TYPE:
                     # Small datetime
                     # 2.2.5.5.1.8
                     # Date/Times
@@ -938,7 +1157,7 @@ class MSSQL():
                 value = struct.unpack('<l',data[:struct.calcsize('<l')])[0]
                 data = data[struct.calcsize('<l'):]
 
-            elif (_type == TDS_FLTNTYPE):
+            elif _type == TDS_FLTNTYPE:
                 valueSize = ord(data[:1])
                 if valueSize == 4:
                     fmt = '<f'
@@ -988,12 +1207,12 @@ class MSSQL():
                 data = data[struct.calcsize('<q'):]
 
 
-            elif (_type == TDS_INT2TYPE):
+            elif _type == TDS_INT2TYPE:
                 #print "INT2TYPE"
                 value = struct.unpack('<H',(data[:2]))[0]
                 data = data[2:]
 
-            elif (_type == TDS_DATENTYPE):
+            elif _type == TDS_DATENTYPE:
                 # date is represented as one 3-byte unsigned integer that represents the number of days since
                 # January 1, year 1.
                 valueSize = ord(data[:1])
@@ -1039,7 +1258,7 @@ class MSSQL():
                 else:
                     value = 'NULL'
 
-            elif (_type == TDS_BITNTYPE):
+            elif _type == TDS_BITNTYPE:
                 #print "BITNTYPE"
                 valueSize = ord(data[:1])
                 data = data[1:]
@@ -1052,7 +1271,7 @@ class MSSQL():
                     value = 'NULL'
                 data = data[valueSize:]
 
-            elif (_type == TDS_INTNTYPE):
+            elif _type == TDS_INTNTYPE:
                 valueSize = ord(data[:1])
                 if valueSize == 1:
                     fmt = '<B'
@@ -1072,7 +1291,7 @@ class MSSQL():
                     data = data[valueSize:]
                 else:
                     value = 'NULL'
-            elif (_type == TDS_SSVARIANTTYPE):
+            elif _type == TDS_SSVARIANTTYPE:
                 LOG.critical("ParseRow: SQL Variant type not yet supported :(")
                 raise
             else:
@@ -1087,7 +1306,7 @@ class MSSQL():
 
         self.rows.append(row)
 
-        return (origDataLen - len(data))
+        return origDataLen - len(data)
 
     def parseColMetaData(self, token):
         # TODO Add support for more data types!
@@ -1130,7 +1349,7 @@ class MSSQL():
                 typeData = ord(data[0])
                 data = data[1:]
 
-            elif (colType == TDS_DATETIMNTYPE): 
+            elif colType == TDS_DATETIMNTYPE:
                 # For DATETIMNTYPE, the only valid lengths are 0x04 and 0x08, which map to smalldatetime and
                 # datetime SQL data types respectively.
                 typeData = ord(data[0])
@@ -1192,7 +1411,7 @@ class MSSQL():
             column['Flags'] = flags
             self.colMeta.append(column)
 
-        return (origDataLen - len(data))
+        return origDataLen - len(data)
 
     def parseReply(self, tokens,tuplemode=False):
         if len(tokens) == 0:
