@@ -52,21 +52,17 @@
 #
 ################################################################################
 
-import sys
 import unittest
 import ConfigParser
 from struct import pack, unpack
+from binascii import unhexlify
 
 from impacket.dcerpc.v5 import transport
 from impacket.dcerpc.v5 import epm, nrpc
-from impacket.dcerpc.v5.ndr import NULL
-from impacket.dcerpc.v5.rpcrt import RPC_C_AUTHN_LEVEL_PKT_INTEGRITY,RPC_C_AUTHN_NETLOGON
-from impacket.dcerpc.v5.dtypes import *
-from impacket.winregistry import hexdump
-from impacket.uuid import string_to_bin, uuidtup_to_bin
-from impacket import system_errors
+from impacket.dcerpc.v5.dtypes import NULL
 from impacket import ntlm
 
+
 class NRPCTests(unittest.TestCase):
     def connect(self):
         rpctransport = transport.DCERPCTransportFactory(self.stringBinding)
@@ -89,7 +85,7 @@ class NRPCTests(unittest.TestCase):
         if self.hashes == '':
             ntHash = None
         else:
-            ntHash = self.hashes.split(':')[1].decode('hex')
+            ntHash = unhexlify(self.hashes.split(':')[1])
 
         self.sessionKey = nrpc.ComputeSessionKeyStrongKey(self.password, '12345678', serverChallenge, ntHash)
 
@@ -317,7 +313,7 @@ class NRPCTests(unittest.TestCase):
         if self.hashes == '':
             ntHash = None
         else:
-            ntHash = self.hashes.split(':')[1].decode('hex')
+            ntHash = unhexlify(self.hashes.split(':')[1])
 
         sessionKey = nrpc.ComputeSessionKeyStrongKey(self.password, '12345678', serverChallenge, ntHash)
 
@@ -343,7 +339,7 @@ class NRPCTests(unittest.TestCase):
         if self.hashes == '':
             ntHash = None
         else:
-            ntHash = self.hashes.split(':')[1].decode('hex')
+            ntHash = unhexlify(self.hashes.split(':')[1])
 
         sessionKey = nrpc.ComputeSessionKeyStrongKey(self.password, '12345678', serverChallenge, ntHash)
 
@@ -367,7 +363,7 @@ class NRPCTests(unittest.TestCase):
         if self.hashes == '':
             ntHash = None
         else:
-            ntHash = self.hashes.split(':')[1].decode('hex')
+            ntHash = unhexlify(self.hashes.split(':')[1])
 
         sessionKey = nrpc.ComputeSessionKeyStrongKey(self.password, '12345678', serverChallenge, ntHash)
 
@@ -386,7 +382,7 @@ class NRPCTests(unittest.TestCase):
         if self.hashes == '':
             ntHash = None
         else:
-            ntHash = self.hashes.split(':')[1].decode('hex')
+            ntHash = unhexlify(self.hashes.split(':')[1])
 
         sessionKey = nrpc.ComputeSessionKeyStrongKey(self.password, '12345678', serverChallenge, ntHash)
 
@@ -417,7 +413,7 @@ class NRPCTests(unittest.TestCase):
         if self.hashes == '':
             ntHash = None
         else:
-            ntHash = self.hashes.split(':')[1].decode('hex')
+            ntHash = unhexlify(self.hashes.split(':')[1])
 
         sessionKey = nrpc.ComputeSessionKeyStrongKey(self.password, '12345678', serverChallenge, ntHash)
 
@@ -446,7 +442,7 @@ class NRPCTests(unittest.TestCase):
         if self.hashes == '':
             ntHash = None
         else:
-            ntHash = self.hashes.split(':')[1].decode('hex')
+            ntHash = unhexlify(self.hashes.split(':')[1])
 
         sessionKey = nrpc.ComputeSessionKeyStrongKey(self.password, '12345678', serverChallenge, ntHash)
 
@@ -557,8 +553,8 @@ class NRPCTests(unittest.TestCase):
         request['LogonInformation']['LogonInteractive']['Identity']['Workstation'] = ''
         if len(self.hashes) > 0:
             lmhash, nthash = self.hashes.split(':')
-            lmhash = lmhash.decode('hex')
-            nthash = nthash.decode('hex')
+            lmhash = unhexlify(lmhash)
+            nthash = unhexlify(nthash)
         else:
             lmhash = ntlm.LMOWFv1(self.password)
             nthash = ntlm.NTOWFv1(self.password)
@@ -568,7 +564,6 @@ class NRPCTests(unittest.TestCase):
             print "Warning: You don't have any crypto installed. You need PyCrypto"
             print "See http://www.pycrypto.org/"
 
-        from impacket import crypto
         rc4 = ARC4.new(self.sessionKey)
         lmhash = rc4.encrypt(lmhash)
         rc4 = ARC4.new(self.sessionKey)
@@ -594,8 +589,8 @@ class NRPCTests(unittest.TestCase):
         request['LogonInformation']['LogonInteractive']['Identity']['Workstation'] = ''
         if len(self.hashes) > 0:
             lmhash, nthash = self.hashes.split(':')
-            lmhash = lmhash.decode('hex')
-            nthash = nthash.decode('hex')
+            lmhash = unhexlify(lmhash)
+            nthash = unhexlify(nthash)
         else:
             lmhash = ntlm.LMOWFv1(self.password)
             nthash = ntlm.NTOWFv1(self.password)
@@ -606,7 +601,6 @@ class NRPCTests(unittest.TestCase):
             print "Warning: You don't have any crypto installed. You need PyCrypto"
             print "See http://www.pycrypto.org/"
 
-        from impacket import crypto
         rc4 = ARC4.new(self.sessionKey)
         lmhash = rc4.encrypt(lmhash)
         rc4 = ARC4.new(self.sessionKey)
@@ -639,8 +633,8 @@ class NRPCTests(unittest.TestCase):
         request['LogonInformation']['LogonInteractive']['Identity']['Workstation'] = ''
         if len(self.hashes) > 0:
             lmhash, nthash = self.hashes.split(':')
-            lmhash = lmhash.decode('hex')
-            nthash = nthash.decode('hex')
+            lmhash = unhexlify(lmhash)
+            nthash = unhexlify(nthash)
         else:
             lmhash = ntlm.LMOWFv1(self.password)
             nthash = ntlm.NTOWFv1(self.password)
@@ -651,7 +645,6 @@ class NRPCTests(unittest.TestCase):
             print "Warning: You don't have any crypto installed. You need PyCrypto"
             print "See http://www.pycrypto.org/"
 
-        from impacket import crypto
         rc4 = ARC4.new(self.sessionKey)
         lmhash = rc4.encrypt(lmhash)
         rc4 = ARC4.new(self.sessionKey)
