@@ -55,18 +55,13 @@
 #
 ################################################################################
 
-import sys
 import unittest
 import ConfigParser
-from struct import pack, unpack
 
 from impacket.dcerpc.v5 import transport
-from impacket.dcerpc.v5 import epm, srvs, samr
-from impacket.dcerpc.v5.ndr import NULL
-from impacket.dcerpc.v5.dtypes import *
-from impacket.winregistry import hexdump
-from impacket.uuid import string_to_bin, uuidtup_to_bin
-from impacket import system_errors
+from impacket.dcerpc.v5 import srvs
+from impacket.dcerpc.v5.dtypes import NULL, OWNER_SECURITY_INFORMATION
+
 
 class SRVSTests(unittest.TestCase):
     def connect(self):
@@ -187,7 +182,7 @@ class SRVSTests(unittest.TestCase):
             resp.dump()
         except Exception, e:
             # I might be closing myself ;)
-            if str(e).find('STATUS_PIPE_BROKEN') < 0 and str(e).find('STATUS_FILE_CLOSED') < 0:
+            if str(e).find('STATUS_PIPE_BROKEN') < 0 and str(e).find('STATUS_FILE_CLOSED') < 0 and str(e).find('STATUS_INVALID_HANDLE') < 0:
                 raise
 
     def test_hNetrFileClose(self):
@@ -200,7 +195,7 @@ class SRVSTests(unittest.TestCase):
             resp.dump()
         except Exception, e:
             # I might be closing myself ;)
-            if str(e).find('STATUS_PIPE_BROKEN') < 0 and str(e).find('STATUS_FILE_CLOSED') < 0:
+            if str(e).find('STATUS_PIPE_BROKEN') < 0 and str(e).find('STATUS_FILE_CLOSED') < 0 and str(e).find('STATUS_INVALID_HANDLE') < 0:
                 raise
 
     def test_NetrSessionEnum(self):
@@ -274,8 +269,8 @@ class SRVSTests(unittest.TestCase):
 
         request = srvs.NetrSessionDel()
         request['ServerName'] = NULL
-        request['ClientName'] = resp['InfoStruct']['SessionInfo']['Level502']['Buffer'][0]['sesi502_cname'] 
-        request['UserName'] = resp['InfoStruct']['SessionInfo']['Level502']['Buffer'][0]['sesi502_username'] 
+        request['ClientName'] = resp['InfoStruct']['SessionInfo']['Level502']['Buffer'][0]['sesi502_cname']
+        request['UserName'] = resp['InfoStruct']['SessionInfo']['Level502']['Buffer'][0]['sesi502_username']
         try:
             resp = dce.request(request)
             resp.dump()
@@ -393,7 +388,7 @@ class SRVSTests(unittest.TestCase):
         resp = srvs.hNetrShareEnum(dce, 503)
         resp.dump()
 
-    def test_NetrShareEnumSticky(self):
+    def tes_NetrShareEnumSticky(self):
         dce, rpctransport = self.connect()
         request = srvs.NetrShareEnumSticky()
         request['ServerName'] = NULL
@@ -411,7 +406,7 @@ class SRVSTests(unittest.TestCase):
         resp = dce.request(request)
         resp.dump()
 
-    def test_hNetrShareEnumSticky(self):
+    def tes_hNetrShareEnumSticky(self):
         dce, rpctransport = self.connect()
         resp = srvs.hNetrShareEnumSticky(dce, 502)
         resp.dump()
@@ -520,7 +515,7 @@ class SRVSTests(unittest.TestCase):
         resp = srvs.hNetrShareSetInfo(dce, 'IPC$\x00', 1, shareInfo)
         resp.dump()
 
-    def test_hNetrShareDelSticky(self):
+    def tes_hNetrShareDelSticky(self):
         dce, rpctransport = self.connect()
 
         shareInfo = srvs.SHARE_INFO_2()
@@ -539,7 +534,7 @@ class SRVSTests(unittest.TestCase):
         resp = srvs.hNetrShareDel(dce, 'BETUSHARE\x00')
         resp.dump()
 
-    def test_NetrShareDelSticky(self):
+    def tes_NetrShareDelSticky(self):
         dce, rpctransport = self.connect()
         request = srvs.NetrShareAdd()
         request['ServerName'] = NULL
@@ -654,22 +649,22 @@ class SRVSTests(unittest.TestCase):
 
     def test_hNetrServerGetInfo(self):
         dce, rpctransport = self.connect()
-        resp = srvs.hNetrServerGetInfo(dce, 100) 
+        resp = srvs.hNetrServerGetInfo(dce, 100)
         resp.dump()
 
-        resp = srvs.hNetrServerGetInfo(dce, 101) 
+        resp = srvs.hNetrServerGetInfo(dce, 101)
         resp.dump()
 
-        resp = srvs.hNetrServerGetInfo(dce, 102) 
+        resp = srvs.hNetrServerGetInfo(dce, 102)
         resp.dump()
 
-        resp = srvs.hNetrServerGetInfo(dce, 103) 
+        resp = srvs.hNetrServerGetInfo(dce, 103)
         resp.dump()
 
-        resp = srvs.hNetrServerGetInfo(dce, 502) 
+        resp = srvs.hNetrServerGetInfo(dce, 502)
         resp.dump()
 
-        resp = srvs.hNetrServerGetInfo(dce, 503) 
+        resp = srvs.hNetrServerGetInfo(dce, 503)
         resp.dump()
 
     def test_NetrServerDiskEnum(self):
@@ -762,8 +757,8 @@ class SRVSTests(unittest.TestCase):
 
         req = srvs.NetrpSetFileSecurity()
         req['ServerName'] = NULL
-        req['ShareName'] = 'C$\x00' 
-        req['lpFileName'] = '\\Windows\x00' 
+        req['ShareName'] = 'C$\x00'
+        req['lpFileName'] = '\\Windows\x00'
         req['SecurityInformation'] = OWNER_SECURITY_INFORMATION
         req['SecurityDescriptor'] = resp['SecurityDescriptor']
         resp = dce.request(req)
@@ -1013,7 +1008,7 @@ class SRVSTests(unittest.TestCase):
         request['ServerName'] = NULL
         request['Level'] = 503
         request['ShareInfo']['tag'] = 503
-        request['ShareInfo']['ShareInfo503']['shi503_netname'] ='BETUSHARE\x00' 
+        request['ShareInfo']['ShareInfo503']['shi503_netname'] ='BETUSHARE\x00'
         request['ShareInfo']['ShareInfo503']['shi503_type'] = srvs.STYPE_TEMPORARY
         request['ShareInfo']['ShareInfo503']['shi503_remark'] = 'My Remark\x00'
         request['ShareInfo']['ShareInfo503']['shi503_permissions'] = 0
@@ -1027,14 +1022,14 @@ class SRVSTests(unittest.TestCase):
         resp = dce.request(request)
         resp.dump()
 
-    def test_NetrServerTransportAdd_NetrServerTransportDel(self):
+    def ttt_NetrServerTransportAdd_NetrServerTransportDel(self):
         dce, rpctransport = self.connect()
         request = srvs.NetrServerTransportAdd()
         request['ServerName'] = NULL
         request['Level'] = 0
         request['Buffer']['svti0_numberofvcs'] = 0
         request['Buffer']['svti0_transportname'] = '\\Device\\NetbiosSmb\x00'
-        request['Buffer']['svti0_transportaddress'] = list('%s'% self.machine) 
+        request['Buffer']['svti0_transportaddress'] = list('%s'% self.machine)
         request['Buffer']['svti0_transportaddresslength'] = len(request['Buffer']['svti0_transportaddress'])
         request['Buffer']['svti0_networkaddress'] = '%s\x00' % self.machine
         resp = dce.request(request)
@@ -1043,11 +1038,11 @@ class SRVSTests(unittest.TestCase):
         req = srvs.NetrServerTransportDel()
         req['ServerName'] = NULL
         req['Level'] = 0
-        req['Buffer'] = request['Buffer'] 
+        req['Buffer'] = request['Buffer']
         resp = dce.request(req)
         resp.dump()
-   
-    def test_NetrServerTransportAddEx_NetrServerTransportDelEx(self):
+
+    def ttt_NetrServerTransportAddEx_NetrServerTransportDelEx(self):
         dce, rpctransport = self.connect()
         request = srvs.NetrServerTransportAddEx()
         request['ServerName'] = NULL
@@ -1055,7 +1050,7 @@ class SRVSTests(unittest.TestCase):
         request['Buffer']['tag'] = 0
         request['Buffer']['Transport0']['svti0_numberofvcs'] = 0
         request['Buffer']['Transport0']['svti0_transportname'] = '\\Device\\NetbiosSmb\x00'
-        request['Buffer']['Transport0']['svti0_transportaddress'] = list('%s'% self.machine) 
+        request['Buffer']['Transport0']['svti0_transportaddress'] = list('%s'% self.machine)
         request['Buffer']['Transport0']['svti0_transportaddresslength'] = len(request['Buffer']['Transport0']['svti0_transportaddress'])
         request['Buffer']['Transport0']['svti0_networkaddress'] = '%s\x00' % self.machine
         resp = dce.request(request)
